@@ -1,22 +1,33 @@
 #pragma once
 
 #ifdef _WIN32
+#define NOMINMAX
 #include <Windows.h>
 #elif defined(__APPLE__)
+#include <TargetConditionals.h>
 #include <mach/mach.h>
 #include <thread>
 #elif defined(__unix__)
 #include <signal.h>
 #endif
 
+#include <optional>
+
 #include "../GenericMipsExecutor.h"
 
 class CEeExecutor : public CGenericMipsExecutor<BlockLookupTwoWay>
 {
 public:
+	using CachedBlockKey = std::pair<uint128, uint32>;
+	using IdleLoopBlockMap = std::map<uint32, std::optional<CachedBlockKey>>;
+	using BlockFpRoundingModeMap = std::map<uint32, Jitter::CJitter::ROUNDINGMODE>;
+
 	CEeExecutor(CMIPS&, uint8*);
 	virtual ~CEeExecutor() = default;
 
+	void SetBlockFpRoundingModes(BlockFpRoundingModeMap);
+	void SetIdleLoopBlocks(IdleLoopBlockMap);
+
 	void AddExceptionHandler();
 	void RemoveExceptionHandler();
 
@@ -28,10 +39,12 @@ public:
 	BasicBlockPtr BlockFactory(CMIPS&, uint32, uint32) override;
 
 private:
-	typedef std::pair<uint128, uint32> CachedBlockKey;
 	typedef std::map<CachedBlockKey, BasicBlockPtr> CachedBlockMap;
 	CachedBlockMap m_cachedBlocks;
 
+	IdleLoopBlockMap m_idleLoopBlocks;
+	BlockFpRoundingModeMap m_blockFpRoundingModes;
+
 	uint8* m_ram = nullptr;
 	size_t m_pageSize = 0;
 
@@ -46,7 +59,7 @@ private:
 #elif defined(__unix__) || defined(__ANDROID__)
 	static void HandleException(int, siginfo_t*, void*);
 	void HandleExceptionInternal(int, siginfo_t*, void*);
-#elif defined(__APPLE__)
+#elif defined(__APPLE__) && !TARGET_OS_TV
 	void HandlerThreadProc();
 
 	mach_port_t m_port = MACH_PORT_NULL;
