@@ -848,6 +848,60 @@ void CMA_EE::PMINW()
 	PullVector(m_nRD);
 }
 
+//05
+void CMA_EE::PABSH()
+{
+	if(m_nRD == 0) return;
+
+	//RD = (RT != 0x8000) ? |RT| : 0x7FFF;
+
+	//Generate mask for overflow
+	m_codeGen->MD_PushCstExpandW(0x80008000U);
+	m_codeGen->MD_PushRel(offsetof(CMIPS, m_State.nGPR[m_nRT]));
+	m_codeGen->MD_CmpEqH();
+	uint32 isOverflowMaskCursor = m_codeGen->GetTopCursor();
+
+	//Generate mask for positive number
+	m_codeGen->MD_PushRel(offsetof(CMIPS, m_State.nGPR[m_nRT]));
+	m_codeGen->MD_PushCstExpandW(0xFFFFFFFFU);
+	m_codeGen->MD_CmpGtH();
+	uint32 isPositiveMaskCursor = m_codeGen->GetTopCursor();
+
+	//Compute neg(rt)
+	m_codeGen->MD_PushCstExpandW(0U);
+	m_codeGen->MD_PushRel(offsetof(CMIPS, m_State.nGPR[m_nRT]));
+	m_codeGen->MD_SubH();
+
+	//Mask result with not(isPositiveMaskCursor)
+	m_codeGen->PushCursor(isPositiveMaskCursor);
+	m_codeGen->MD_Not();
+	m_codeGen->MD_And();
+
+	//Mask input with isPositiveMaskCursor and combine with neg(rt)
+	m_codeGen->MD_PushRel(offsetof(CMIPS, m_State.nGPR[m_nRT]));
+	m_codeGen->PushCursor(isPositiveMaskCursor);
+	m_codeGen->MD_And();
+	m_codeGen->MD_Or();
+
+	//Mask result with not(isOverflowMaskCursor)
+	m_codeGen->PushCursor(isOverflowMaskCursor);
+	m_codeGen->MD_Not();
+	m_codeGen->MD_And();
+
+	//Combine with overflow
+	m_codeGen->PushCursor(isOverflowMaskCursor);
+	m_codeGen->MD_PushCstExpandW(0x7FFF7FFFU);
+	m_codeGen->MD_And();
+	m_codeGen->MD_Or();
+
+	//Save result
+	m_codeGen->MD_PullRel(offsetof(CMIPS, m_State.nGPR[m_nRD]));
+
+	//Discard our masks
+	m_codeGen->PullTop();
+	m_codeGen->PullTop();
+}
+
 //06
 void CMA_EE::PCEQH()
 {
@@ -1196,25 +1250,25 @@ void CMA_EE::PHMADH()
 
 	for(unsigned int i = 0; i < 4; i++)
 	{
-		//Lower 16-bits (An0 * An1)
+		//Higher 16-bits (A(2i + 1) * B(2i + 1))
 		{
 			m_codeGen->PushRel(offsetof(CMIPS, m_State.nGPR[m_nRS].nV[i]));
-			m_codeGen->SignExt16();
+			m_codeGen->Sra(16);
 
 			m_codeGen->PushRel(offsetof(CMIPS, m_State.nGPR[m_nRT].nV[i]));
-			m_codeGen->SignExt16();
+			m_codeGen->Sra(16);
 
 			m_codeGen->MultS();
 			m_codeGen->ExtLow64();
 		}
 
-		//Higher 16-bits (Bn0 * Bn1)
+		//Lower 16-bits (A(2i) * B(2i))
 		{
 			m_codeGen->PushRel(offsetof(CMIPS, m_State.nGPR[m_nRS].nV[i]));
-			m_codeGen->Sra(16);
+			m_codeGen->SignExt16();
 
 			m_codeGen->PushRel(offsetof(CMIPS, m_State.nGPR[m_nRT].nV[i]));
-			m_codeGen->Sra(16);
+			m_codeGen->SignExt16();
 
 			m_codeGen->MultS();
 			m_codeGen->ExtLow64();
@@ -1255,6 +1309,70 @@ void CMA_EE::PXOR()
 	PullVector(m_nRD);
 }
 
+//15
+void CMA_EE::PHMSBH()
+{
+	static const size_t offsets[4] =
+	    {
+	        offsetof(CMIPS, m_State.nLO[0]),
+	        offsetof(CMIPS, m_State.nHI[0]),
+	        offsetof(CMIPS, m_State.nLO1[0]),
+	        offsetof(CMIPS, m_State.nHI1[0]),
+	    };
+
+	static const size_t clearOffsets[4] =
+	    {
+	        offsetof(CMIPS, m_State.nLO[1]),
+	        offsetof(CMIPS, m_State.nHI[1]),
+	        offsetof(CMIPS, m_State.nLO1[1]),
+	        offsetof(CMIPS, m_State.nHI1[1]),
+	    };
+
+	for(unsigned int i = 0; i < 4; i++)
+	{
+		m_codeGen->PushCst(0);
+		m_codeGen->PullRel(clearOffsets[i]);
+	}
+
+	for(unsigned int i = 0; i < 4; i++)
+	{
+		//Higher 16-bits (A(2i + 1) * B(2i + 1))
+		{
+			m_codeGen->PushRel(offsetof(CMIPS, m_State.nGPR[m_nRS].nV[i]));
+			m_codeGen->Sra(16);
+
+			m_codeGen->PushRel(offsetof(CMIPS, m_State.nGPR[m_nRT].nV[i]));
+			m_codeGen->Sra(16);
+
+			m_codeGen->MultS();
+			m_codeGen->ExtLow64();
+		}
+
+		//Lower 16-bits (A(2i) * B(2i))
+		{
+			m_codeGen->PushRel(offsetof(CMIPS, m_State.nGPR[m_nRS].nV[i]));
+			m_codeGen->SignExt16();
+
+			m_codeGen->PushRel(offsetof(CMIPS, m_State.nGPR[m_nRT].nV[i]));
+			m_codeGen->SignExt16();
+
+			m_codeGen->MultS();
+			m_codeGen->ExtLow64();
+		}
+
+		m_codeGen->Sub();
+
+		if(m_nRD != 0)
+		{
+			m_codeGen->PushTop();
+			m_codeGen->PullRel(offsetof(CMIPS, m_State.nGPR[m_nRD].nV[i]));
+		}
+
+		//Store to LO/HI
+		m_codeGen->PullRel(offsets[i]);
+	}
+}
+
 //1A
 void CMA_EE::PEXEH()
 {
@@ -2035,7 +2153,7 @@ CMA_EE::InstructionFuncConstant CMA_EE::m_pOpMmi0[0x20] =
 CMA_EE::InstructionFuncConstant CMA_EE::m_pOpMmi1[0x20] = 
 {
 	//0x00
-	&CMA_EE::Illegal,		&CMA_EE::PABSW,			&CMA_EE::PCEQW,			&CMA_EE::PMINW,			&CMA_EE::Illegal,		&CMA_EE::Illegal,		&CMA_EE::PCEQH,			&CMA_EE::PMINH,
+	&CMA_EE::Illegal,		&CMA_EE::PABSW,			&CMA_EE::PCEQW,			&CMA_EE::PMINW,			&CMA_EE::Illegal,		&CMA_EE::PABSH,			&CMA_EE::PCEQH,			&CMA_EE::PMINH,
 	//0x08
 	&CMA_EE::Illegal,		&CMA_EE::Illegal,		&CMA_EE::PCEQB,			&CMA_EE::Illegal,		&CMA_EE::Illegal,		&CMA_EE::Illegal,		&CMA_EE::Illegal,		&CMA_EE::Illegal,
 	//0x10
@@ -2051,7 +2169,7 @@ CMA_EE::InstructionFuncConstant CMA_EE::m_pOpMmi2[0x20] =
 	//0x08
 	&CMA_EE::PMFHI,			&CMA_EE::PMFLO,			&CMA_EE::PINTH,			&CMA_EE::Illegal,		&CMA_EE::PMULTW,		&CMA_EE::PDIVW,			&CMA_EE::PCPYLD,		&CMA_EE::Illegal,
 	//0x10
-	&CMA_EE::PMADDH,		&CMA_EE::PHMADH,		&CMA_EE::PAND,			&CMA_EE::PXOR,			&CMA_EE::Illegal,		&CMA_EE::Illegal,		&CMA_EE::Illegal,		&CMA_EE::Illegal,
+	&CMA_EE::PMADDH,		&CMA_EE::PHMADH,		&CMA_EE::PAND,			&CMA_EE::PXOR,			&CMA_EE::Illegal,		&CMA_EE::PHMSBH,		&CMA_EE::Illegal,		&CMA_EE::Illegal,
 	//0x18
 	&CMA_EE::Illegal,		&CMA_EE::Illegal,		&CMA_EE::PEXEH,			&CMA_EE::PREVH,			&CMA_EE::PMULTH,		&CMA_EE::PDIVBW,		&CMA_EE::PEXEW,			&CMA_EE::PROT3W,
 };
