@@ -9,6 +9,11 @@ CVuBasicBlock::CVuBasicBlock(CMIPS& context, uint32 begin, uint32 end, BLOCK_CAT
 {
 }
 
+void CVuBasicBlock::AddBlockCompileHints(uint32 compileHints)
+{
+	m_blockCompileHints |= compileHints;
+}
+
 bool CVuBasicBlock::IsLinkable() const
 {
 	return m_isLinkable;
@@ -35,6 +40,8 @@ void CVuBasicBlock::CompileRange(CMipsJitter* jitter)
 
 	auto integerBranchDelayInfo = ComputeIntegerBranchDelayInfo(fmacPipelineInfo.stallDelays);
 
+	auto integerTrailingDelayInfo = ComputeTrailingIntegerBranchDelayInfo(fmacPipelineInfo.stallDelays);
+
 	uint32 maxInstructions = ((m_end - m_begin) / 8) + 1;
 	std::vector<uint32> hints;
 	hints.resize(maxInstructions);
@@ -97,7 +104,7 @@ void CVuBasicBlock::CompileRange(CMipsJitter* jitter)
 						jitter->BeginIf(Jitter::CONDITION_NE);
 						{
 							//Clear writes
-							jitter->MD_PushCstExpand(0U);
+							jitter->MD_PushCstExpandW(0U);
 							jitter->MD_PullRel(offsetof(CMIPS, m_State.pipeFmacWrite[i]));
 
 							//Increment pipe time
@@ -112,7 +119,7 @@ void CVuBasicBlock::CompileRange(CMipsJitter* jitter)
 			}
 
 			//Clear writes
-			jitter->MD_PushCstExpand(0U);
+			jitter->MD_PushCstExpandW(0U);
 			jitter->MD_PullRel(offsetof(CMIPS, m_State.pipeFmacWrite[extraPipeTimeIndex]));
 		}
 		extraPipeTimeIndex++;
@@ -159,8 +166,14 @@ void CVuBasicBlock::CompileRange(CMipsJitter* jitter)
 			jitter->PushRel(offsetof(CMIPS, m_State.nCOP2VI[integerBranchDelayInfo.regIndex]));
 			jitter->PullRel(offsetof(CMIPS, m_State.savedIntReg));
 		}
+		if(address == integerTrailingDelayInfo.saveRegAddress)
+		{
+			// grab the value of the delayed reg to possibly use in the next block
+			jitter->PushRel(offsetof(CMIPS, m_State.nCOP2VI[integerTrailingDelayInfo.regIndex]));
+			jitter->PullRel(offsetof(CMIPS, m_State.savedNextBlockIntRegVal));
+		}
 
-		uint32 compileHints = hints[instructionIndex];
+		uint32 compileHints = hints[instructionIndex] | m_blockCompileHints;
 		arch->SetRelativePipeTime(relativePipeTime, compileHints);
 		arch->CompileInstruction(addressHi, jitter, &m_context, addressHi - m_begin);
 
@@ -183,6 +196,58 @@ void CVuBasicBlock::CompileRange(CMipsJitter* jitter)
 			jitter->PullRel(offsetof(CMIPS, m_State.nCOP2VI[integerBranchDelayInfo.regIndex]));
 		}
 
+		if(address == m_begin)
+		{
+			if(IsConditionalBranch(opcodeLo))
+			{
+				auto branchLoOps = arch->GetAffectedOperands(&m_context, address, opcodeLo);
+				// Block starts with a conditional branch.
+				// Need to look in state to see if there was an integer altering operation as the last thing in the prior block.
+				// If there was, then check if the integer register matches.
+				// If it matches then use the reg value stored in the state.
+
+				// This needs to be done at runtime (i.e. we generate some jit code now to do that logic)
+				// set the target from the saved value
+
+				/*
+				    jit code needs to do something like follows:
+
+				    if(branchLoOps.readI0 == m_State.savedNextBlockIntRegIdx || branchLoOps.readI1 == m_State.savedNextBlockIntRegIdx)
+				    {
+				        m_State.savedNextBlockIntRegValTemp = m_State.nCOP2VI[m_State.savedNextBlockIntRegIdx];
+				        m_State.nCOP2VI[m_State.savedNextBlockIntRegIdx] = m_State.savedNextBlockIntRegVal;
+				    }
+				 */
+				//m_State.savedNextBlockIntRegIdx == branchLoOps.readI0
+				jitter->PushRel(offsetof(CMIPS, m_State.savedNextBlockIntRegIdx));
+				jitter->PushCst(branchLoOps.readI0);
+				jitter->Cmp(Jitter::CONDITION_EQ);
+
+				//m_State.savedNextBlockIntRegIdx == branchLoOps.readI1
+				jitter->PushRel(offsetof(CMIPS, m_State.savedNextBlockIntRegIdx));
+				jitter->PushCst(branchLoOps.readI1);
+				jitter->Cmp(Jitter::CONDITION_EQ);
+
+				jitter->Or();
+				jitter->PushCst(0);
+				jitter->BeginIf(Jitter::CONDITION_NE);
+				{
+					// m_State.savedNextBlockIntRegValTemp = m_State.nCOP2VI[m_State.savedNextBlockIntRegIdx];
+					jitter->PushRelAddrRef(offsetof(CMIPS, m_State.nCOP2VI));
+					jitter->PushRel(offsetof(CMIPS, m_State.savedNextBlockIntRegIdx));
+					jitter->LoadFromRefIdx();
+					jitter->PullRel(offsetof(CMIPS, m_State.savedNextBlockIntRegValTemp));
+
+					// m_State.nCOP2VI[m_State.savedNextBlockIntRegIdx] = m_State.savedNextBlockIntRegVal;
+					jitter->PushRelAddrRef(offsetof(CMIPS, m_State.nCOP2VI));
+					jitter->PushRel(offsetof(CMIPS, m_State.savedNextBlockIntRegIdx));
+					jitter->PushRel(offsetof(CMIPS, m_State.savedNextBlockIntRegVal));
+					jitter->StoreAtRefIdx();
+				}
+				jitter->EndIf();
+			}
+		}
+
 		//If there's a pending XGKICK and the current lower instruction is
 		//an XGKICK, make sure we flush the pending one first
 		if(loIsXgKick && hasPendingXgKick)
@@ -199,6 +264,44 @@ void CVuBasicBlock::CompileRange(CMipsJitter* jitter)
 			jitter->PullRel(offsetof(CMIPS, m_State.nCOP2VI[integerBranchDelayInfo.regIndex]));
 		}
 
+		if(address == m_begin)
+		{
+			if(IsConditionalBranch(opcodeLo))
+			{
+				/*
+				    jit code needs to do something like follows:
+
+				    if(branchLoOps.readI0 == m_State.savedNextBlockIntRegIdx || branchLoOps.readI1 == m_State.savedNextBlockIntRegIdx)
+				    {
+				        m_State.nCOP2VI[m_State.savedNextBlockIntRegIdx] = m_State.savedNextBlockIntRegValTemp;
+				    }
+				 */
+				auto branchLoOps = arch->GetAffectedOperands(&m_context, address, opcodeLo);
+
+				//m_State.savedNextBlockIntRegIdx == branchLoOps.readI0
+				jitter->PushRel(offsetof(CMIPS, m_State.savedNextBlockIntRegIdx));
+				jitter->PushCst(branchLoOps.readI0);
+				jitter->Cmp(Jitter::CONDITION_EQ);
+
+				//m_State.savedNextBlockIntRegIdx == branchLoOps.readI1
+				jitter->PushRel(offsetof(CMIPS, m_State.savedNextBlockIntRegIdx));
+				jitter->PushCst(branchLoOps.readI1);
+				jitter->Cmp(Jitter::CONDITION_EQ);
+
+				jitter->Or();
+				jitter->PushCst(0);
+				jitter->BeginIf(Jitter::CONDITION_NE);
+				{
+					// m_State.nCOP2VI[m_State.savedNextBlockIntRegIdx] = m_State.savedNextBlockIntRegValTemp;
+					jitter->PushRelAddrRef(offsetof(CMIPS, m_State.nCOP2VI));
+					jitter->PushRel(offsetof(CMIPS, m_State.savedNextBlockIntRegIdx));
+					jitter->PushRel(offsetof(CMIPS, m_State.savedNextBlockIntRegValTemp));
+					jitter->StoreAtRefIdx();
+				}
+				jitter->EndIf();
+			}
+		}
+
 		if(savedReg != 0)
 		{
 			jitter->MD_PushRel(offsetof(CMIPS, m_State.nCOP2VF_UpRes));
@@ -227,6 +330,7 @@ void CVuBasicBlock::CompileRange(CMipsJitter* jitter)
 			//Disable block linking because targets will be wrong
 			m_isLinkable = false;
 
+			assert(address >= 8);
 			uint32 branchOpcodeAddr = address - 8;
 			assert(branchOpcodeAddr >= m_begin);
 			uint32 branchOpcodeLo = m_context.m_pMemoryMap->GetInstruction(branchOpcodeAddr);
@@ -238,6 +342,30 @@ void CVuBasicBlock::CompileRange(CMipsJitter* jitter)
 				arch->CompileInstruction(branchTgtAddress, jitter, &m_context, address - m_begin);
 			}
 		}
+		//Handle some E bit in delay slot situation (Edge of Reality games)
+		else if((address == (m_end - 4)) && (opcodeHi & VUShared::VU_UPPEROP_BIT_E))
+		{
+			//Check if we actually had a conditional branch before
+			assert(address >= 8);
+			uint32 branchOpcodeAddr = address - 8;
+			assert(branchOpcodeAddr >= m_begin);
+			uint32 branchOpcodeLo = m_context.m_pMemoryMap->GetInstruction(branchOpcodeAddr);
+			if(IsConditionalBranch(branchOpcodeLo))
+			{
+				//So, this is a E bit in a delay slot. If branch is not taken
+				//we want to execute the instruction in the E bit delay slot.
+				jitter->PushRel(offsetof(CMIPS, m_State.nDelayedJumpAddr));
+				jitter->PushCst(MIPS_INVALID_PC);
+				jitter->BeginIf(Jitter::CONDITION_EQ);
+				{
+					//If we haven't taken the jump, execute the instruction after the delay slot
+					uint32 nextOpcodeAddr = address + 8;
+					uint32 nextOpcodeLo = m_context.m_pMemoryMap->GetInstruction(nextOpcodeAddr);
+					arch->CompileInstruction(nextOpcodeAddr, jitter, &m_context, nextOpcodeAddr - m_begin);
+				}
+				jitter->EndIf();
+			}
+		}
 
 		//Sanity check
 		assert(jitter->IsStackEmpty());
@@ -262,7 +390,7 @@ void CVuBasicBlock::CompileRange(CMipsJitter* jitter)
 	{
 		//Clear unused writes
 		//TODO: Find a way to push that to the next block (would become index 0 of next block)
-		jitter->MD_PushCstExpand(0U);
+		jitter->MD_PushCstExpandW(0U);
 		jitter->MD_PullRel(offsetof(CMIPS, m_State.pipeFmacWrite[i]));
 	}
 
@@ -309,6 +437,9 @@ void CVuBasicBlock::CompileRange(CMipsJitter* jitter)
 		return target == m_begin;
 	}();
 
+	jitter->PushCst(integerTrailingDelayInfo.regIndex);
+	jitter->PullRel(offsetof(CMIPS, m_State.savedNextBlockIntRegIdx));
+
 	CompileEpilog(jitter, loopsOnItself && m_isLinkable);
 }
 
@@ -325,6 +456,32 @@ bool CVuBasicBlock::IsNonConditionalBranch(uint32 opcodeLo)
 	return (id == 0x20);
 }
 
+CVuBasicBlock::INTEGER_BRANCH_DELAY_INFO CVuBasicBlock::ComputeTrailingIntegerBranchDelayInfo(const std::vector<uint32>& fmacStallDelays) const
+{
+	// Test if a block ends with an integer altering instruction.
+	// This is important if the following block starts with a conditional branch
+	// using that same integer register.
+	INTEGER_BRANCH_DELAY_INFO result;
+	auto arch = static_cast<CMA_VU*>(m_context.m_pArch);
+	uint32 adjustedEnd = m_end - 4;
+	uint32 endOpcodeLoAddr = adjustedEnd;
+	uint32 endOpcodeLo = m_context.m_pMemoryMap->GetInstruction(endOpcodeLoAddr);
+	auto endLoOps = arch->GetAffectedOperands(&m_context, endOpcodeLoAddr, endOpcodeLo);
+
+	// assume this is same as for the case if there was a conditional branch at the end.
+	if(fmacStallDelays.size() > 1)
+	{
+		uint32 fmacDelayOnBranch = fmacStallDelays[fmacStallDelays.size() - 2];
+		if((endLoOps.writeI != 0) && !endLoOps.branchValue && (fmacDelayOnBranch == 0))
+		{
+			// we need to save the value of the integer register before the instruction is executed
+			result.regIndex = endLoOps.writeI;
+			result.saveRegAddress = adjustedEnd;
+		}
+	}
+	return result;
+}
+
 CVuBasicBlock::INTEGER_BRANCH_DELAY_INFO CVuBasicBlock::ComputeIntegerBranchDelayInfo(const std::vector<uint32>& fmacStallDelays) const
 {
 	// Test if the block ends with a conditional branch instruction where the condition variable has been
@@ -493,7 +650,7 @@ void CVuBasicBlock::ComputeSkipFlagsHints(const std::vector<uint32>& fmacStallDe
 		bool used = resultUsed[instructionIndex];
 		if(!used)
 		{
-			hints[instructionIndex] |= VUShared::COMPILEHINT_SKIPFMACUPDATE;
+			hints[instructionIndex] |= VUShared::COMPILEHINT_SKIP_FMAC_UPDATE;
 		}
 	}
 }
