@@ -8,6 +8,7 @@ public:
 	CVuBasicBlock(CMIPS&, uint32, uint32, BLOCK_CATEGORY);
 	virtual ~CVuBasicBlock() = default;
 
+	void AddBlockCompileHints(uint32);
 	bool IsLinkable() const;
 
 protected:
@@ -38,10 +39,12 @@ private:
 	};
 
 	INTEGER_BRANCH_DELAY_INFO ComputeIntegerBranchDelayInfo(const std::vector<uint32>&) const;
+	INTEGER_BRANCH_DELAY_INFO ComputeTrailingIntegerBranchDelayInfo(const std::vector<uint32>&) const;
 	bool CheckIsSpecialIntegerLoop(unsigned int) const;
 	void ComputeSkipFlagsHints(const std::vector<uint32>&, std::vector<uint32>&) const;
 	BlockFmacPipelineInfo ComputeFmacStallDelays(uint32, uint32) const;
 	static void EmitXgKick(CMipsJitter*);
 
 	bool m_isLinkable = true;
+	uint32 m_blockCompileHints = 0;
 };
