@@ -26,6 +26,7 @@
 #include "Iop_MtapMan.h"
 #include "Iop_Cdvdfsv.h"
 #include "Iop_McServ.h"
+#include "Iop_Usbd.h"
 #include "Iop_PowerOff.h"
 #endif
 
@@ -93,12 +94,10 @@ public:
 		uint32 threadProc;
 		THREADCONTEXT context;
 		uint32 status;
-		uint32 waitSemaphore;
-		uint32 waitEventFlag;
+		uint32 waitObjectId;
 		uint32 waitEventFlagMode;
 		uint32 waitEventFlagMask;
 		uint32 waitEventFlagResultPtr;
-		uint32 waitMessageBox;
 		uint32 waitMessageBoxResultPtr;
 		uint32 wakeupCount;
 		uint32 stackBase;
@@ -110,14 +109,15 @@ public:
 	enum THREAD_STATUS
 	{
 		THREAD_STATUS_DORMANT = 1,
-		THREAD_STATUS_RUNNING = 2,
-		THREAD_STATUS_SLEEPING = 3,
-		THREAD_STATUS_WAITING_SEMAPHORE = 4,
-		THREAD_STATUS_WAITING_EVENTFLAG = 5,
-		THREAD_STATUS_WAITING_MESSAGEBOX = 6,
-		THREAD_STATUS_WAIT_VBLANK_START = 7,
-		THREAD_STATUS_WAIT_VBLANK_END = 8,
-		THREAD_STATUS_WAIT_CDSYNC = 9,
+		THREAD_STATUS_RUNNING,
+		THREAD_STATUS_SLEEPING,
+		THREAD_STATUS_WAITING_SEMAPHORE,
+		THREAD_STATUS_WAITING_EVENTFLAG,
+		THREAD_STATUS_WAITING_MESSAGEBOX,
+		THREAD_STATUS_WAITING_FPL,
+		THREAD_STATUS_WAIT_VBLANK_START,
+		THREAD_STATUS_WAIT_VBLANK_END,
+		THREAD_STATUS_WAIT_CDSYNC,
 	};
 
 	struct THREAD_INFO
@@ -199,6 +199,7 @@ public:
 
 	void Reset(uint32, const Iop::SifManPtr&);
 
+	void PreLoadState() override;
 	void SaveState(Framework::CZipArchiveWriter&) override;
 	void LoadState(Framework::CZipArchiveReader&) override;
 
@@ -214,6 +215,7 @@ public:
 	Iop::CPadMan* GetPadman();
 	Iop::CCdvdfsv* GetCdvdfsv();
 	Iop::CMcServ* GetMcServ();
+	Iop::CUsbd* GetUsbd();
 #endif
 	bool RegisterModule(const Iop::ModulePtr&);
 	bool ReleaseModule(const std::string&);
@@ -279,6 +281,7 @@ public:
 
 	uint32 CreateVpl(uint32);
 	uint32 DeleteVpl(uint32);
+	uint32 AllocateVpl(uint32, uint32);
 	uint32 pAllocateVpl(uint32, uint32);
 	uint32 FreeVpl(uint32, uint32);
 	uint32 ReferVplStatus(uint32, uint32);
@@ -472,6 +475,7 @@ private:
 		uint32 poolPtr;
 		uint32 blockSize;
 		uint32 blockCount;
+		uint32 waitCount;
 	};
 
 	struct VPL
@@ -619,6 +623,7 @@ private:
 	uint32 LoadExecutable(CELF32&, ExecutableRange&, uint32);
 	unsigned int GetElfProgramToLoad(CELF32&);
 	void RelocateElf(CELF32&, uint32, uint32);
+	std::string_view ReadModuleName(const uint8*);
 	std::string_view ReadModuleName(uint32);
 	void DeleteModules();
 
@@ -699,6 +704,7 @@ private:
 	Iop::McServPtr m_mcserv;
 	Iop::CdvdfsvPtr m_cdvdfsv;
 	Iop::PowerOffPtr m_powerOff;
+	Iop::UsbdPtr m_usbd;
 
 	std::map<std::string, Iop::ModulePtr> m_hleModules;
 #endif
