@@ -1,9 +1,10 @@
 #include <assert.h>
-#include "../Log.h"
+#include "Log.h"
 #include "../Ps2Const.h"
 #include "Iop_Cdvdfsv.h"
 #include "Iop_Cdvdman.h"
 #include "Iop_SifManPs2.h"
+#include "TimeUtils.h"
 
 using namespace Iop;
 
@@ -20,8 +21,11 @@ using namespace Iop;
 #define STATE_STREAMPOS ("StreamPos")
 #define STATE_STREAMBUFFERSIZE ("StreamBufferSize")
 
+constexpr uint64 COMMAND_DEFAULT_DELAY = TimeUtils::UsecsToCycles(PS2::IOP_CLOCK_OVER_FREQ, 16666);
+
 CCdvdfsv::CCdvdfsv(CSifMan& sif, CCdvdman& cdvdman, uint8* iopRam)
-    : m_cdvdman(cdvdman)
+    : m_sifMan(sif)
+    , m_cdvdman(cdvdman)
     , m_iopRam(iopRam)
 {
 	m_module592 = CSifModuleAdapter(std::bind(&CCdvdfsv::Invoke592, this,
@@ -58,66 +62,87 @@ std::string CCdvdfsv::GetFunctionName(unsigned int) const
 	return "unknown";
 }
 
-void CCdvdfsv::ProcessCommands(CSifMan* sifMan)
+void CCdvdfsv::CountTicks(uint32 ticks)
 {
 	if(m_pendingCommand != COMMAND_NONE)
 	{
-		static const uint32 sectorSize = 0x800;
-
-		uint8* eeRam = nullptr;
-		if(auto sifManPs2 = dynamic_cast<CSifManPs2*>(sifMan))
+		m_pendingCommandDelay -= std::min<uint32>(m_pendingCommandDelay, ticks);
+		if(m_pendingCommandDelay != 0)
 		{
-			eeRam = sifManPs2->GetEeRam();
+			return;
 		}
 
-		if(m_pendingCommand == COMMAND_READ)
-		{
-			if(m_opticalMedia != nullptr)
-			{
-				auto fileSystem = m_opticalMedia->GetFileSystem();
-				for(unsigned int i = 0; i < m_pendingReadCount; i++)
-				{
-					fileSystem->ReadBlock(m_pendingReadSector + i, eeRam + (m_pendingReadAddr + (i * sectorSize)));
-				}
-			}
-		}
-		else if(m_pendingCommand == COMMAND_READIOP)
-		{
-			if(m_opticalMedia != nullptr)
-			{
-				auto fileSystem = m_opticalMedia->GetFileSystem();
-				for(unsigned int i = 0; i < m_pendingReadCount; i++)
-				{
-					fileSystem->ReadBlock(m_pendingReadSector + i, m_iopRam + (m_pendingReadAddr + (i * sectorSize)));
-				}
-			}
-		}
-		else if(m_pendingCommand == COMMAND_STREAM_READ)
-		{
-			if(m_opticalMedia != nullptr)
-			{
-				auto fileSystem = m_opticalMedia->GetFileSystem();
-				for(unsigned int i = 0; i < m_pendingReadCount; i++)
-				{
-					fileSystem->ReadBlock(m_streamPos, eeRam + (m_pendingReadAddr + (i * sectorSize)));
-					m_streamPos++;
-				}
-			}
-		}
-		else if(m_pendingCommand == COMMAND_NDISKREADY)
-		{
-			//Result already set, just return normally
-		}
-		else if(m_pendingCommand == COMMAND_READCHAIN)
-		{
-			//Nothing to do, everything has been read already
-		}
-
-		m_pendingCommand = COMMAND_NONE;
-		sifMan->SendCallReply(MODULE_ID_4, nullptr);
+		FinishPendingCommand();
 	}
 }
 
+void CCdvdfsv::FinishPendingCommand()
+{
+	assert(m_pendingCommand != COMMAND_NONE);
+
+	static const uint32 sectorSize = 0x800;
+
+	uint8* eeRam = nullptr;
+	if(auto sifManPs2 = dynamic_cast<CSifManPs2*>(&m_sifMan))
+	{
+		eeRam = sifManPs2->GetEeRam();
+	}
+
+	if(m_pendingCommand == COMMAND_READ)
+	{
+		if(m_opticalMedia != nullptr)
+		{
+			auto fileSystem = m_opticalMedia->GetFileSystem();
+			for(unsigned int i = 0; i < m_pendingReadCount; i++)
+			{
+				fileSystem->ReadBlock(m_pendingReadSector + i, eeRam + (m_pendingReadAddr + (i * sectorSize)));
+			}
+		}
+	}
+	else if(m_pendingCommand == COMMAND_READIOP)
+	{
+		if(m_opticalMedia != nullptr)
+		{
+			auto fileSystem = m_opticalMedia->GetFileSystem();
+			for(unsigned int i = 0; i < m_pendingReadCount; i++)
+			{
+				fileSystem->ReadBlock(m_pendingReadSector + i, m_iopRam + (m_pendingReadAddr + (i * sectorSize)));
+			}
+		}
+	}
+	else if(m_pendingCommand == COMMAND_STREAM_READ)
+	{
+		if(m_opticalMedia != nullptr)
+		{
+			auto fileSystem = m_opticalMedia->GetFileSystem();
+			for(unsigned int i = 0; i < m_pendingReadCount; i++)
+			{
+				fileSystem->ReadBlock(m_streamPos, eeRam + (m_pendingReadAddr + (i * sectorSize)));
+				m_streamPos++;
+			}
+		}
+	}
+	else if(m_pendingCommand == COMMAND_NDISKREADY)
+	{
+		//Result already set, just return normally
+	}
+	else if(m_pendingCommand == COMMAND_READCHAIN)
+	{
+		//Nothing to do, everything has been read already
+	}
+	else if(m_pendingCommand == COMMAND_PAUSE)
+	{
+		//Nothing to do
+	}
+	else
+	{
+		assert(false);
+	}
+
+	m_pendingCommand = COMMAND_NONE;
+	m_sifMan.SendCallReply(MODULE_ID_4, nullptr);
+}
+
 void CCdvdfsv::SetOpticalMedia(COpticalMedia* opticalMedia)
 {
 	m_opticalMedia = opticalMedia;
@@ -296,6 +321,13 @@ bool CCdvdfsv::Invoke595(uint32 method, uint32* args, uint32 argsSize, uint32* r
 	}
 	break;
 
+	case 0x08:
+		CLog::GetInstance().Print(LOG_NAME, "Pause();\r\n");
+		assert(m_pendingCommand == COMMAND_NONE);
+		m_pendingCommand = COMMAND_PAUSE;
+		m_pendingCommandDelay = COMMAND_DEFAULT_DELAY;
+		return false;
+
 	case 0x09:
 		return StreamCmd(args, argsSize, ret, retSize, ram);
 		break;
@@ -371,6 +403,13 @@ bool CCdvdfsv::Invoke59C(uint32 method, uint32* args, uint32 argsSize, uint32* r
 		assert(argsSize >= 4);
 		uint32 mode = args[0x00];
 		CLog::GetInstance().Print(LOG_NAME, "DiskReady(mode = %i);\r\n", mode);
+		//When this is called, it seems we need to make sure that no pending command is
+		//remaining. Castlevania: Curse of Darkness issues a Pause command then uses
+		//DiskReady without acknowledging the pause and assumes it can use GetToc after.
+		if(m_pendingCommand != COMMAND_NONE)
+		{
+			FinishPendingCommand();
+		}
 		ret[0x00] = 2;
 	}
 	break;
@@ -400,6 +439,7 @@ void CCdvdfsv::Read(uint32* args, uint32 argsSize, uint32* ret, uint32 retSize,
 
 	assert(m_pendingCommand == COMMAND_NONE);
 	m_pendingCommand = COMMAND_READ;
+	m_pendingCommandDelay = CCdvdman::COMMAND_READ_BASE_DELAY + (count * CCdvdman::COMMAND_READ_SECTOR_DELAY);
 	m_pendingReadSector = sector;
 	m_pendingReadCount = count;
 	m_pendingReadAddr = dstAddr & 0x1FFFFFFF;
@@ -422,6 +462,7 @@ void CCdvdfsv::ReadIopMem(uint32* args, uint32 argsSize, uint32* ret, uint32 ret
 
 	assert(m_pendingCommand == COMMAND_NONE);
 	m_pendingCommand = COMMAND_READIOP;
+	m_pendingCommandDelay = COMMAND_DEFAULT_DELAY;
 	m_pendingReadSector = sector;
 	m_pendingReadCount = count;
 	m_pendingReadAddr = dstAddr & 0x1FFFFFFF;
@@ -454,6 +495,7 @@ bool CCdvdfsv::StreamCmd(uint32* args, uint32 argsSize, uint32* ret, uint32 retS
 	case 2:
 		//Read
 		m_pendingCommand = COMMAND_STREAM_READ;
+		m_pendingCommandDelay = COMMAND_DEFAULT_DELAY;
 		m_pendingReadSector = 0;
 		m_pendingReadCount = count;
 		m_pendingReadAddr = dstAddr & (PS2::EE_RAM_SIZE - 1);
@@ -509,6 +551,7 @@ bool CCdvdfsv::NDiskReady(uint32* args, uint32 argsSize, uint32* ret, uint32 ret
 	{
 		//Delay command (required by Downhill Domination)
 		m_pendingCommand = COMMAND_NDISKREADY;
+		m_pendingCommandDelay = COMMAND_DEFAULT_DELAY;
 		ret[0x00] = 2;
 		return false;
 	}
@@ -544,6 +587,7 @@ void CCdvdfsv::ReadChain(uint32* args, uint32 argsSize, uint32* ret, uint32 retS
 
 	//DBZ: Budokai Tenkaichi hangs in its loading screen if this command's result is not delayed.
 	m_pendingCommand = COMMAND_READCHAIN;
+	m_pendingCommandDelay = COMMAND_DEFAULT_DELAY;
 }
 
 void CCdvdfsv::SearchFile(uint32* args, uint32 argsSize, uint32* ret, uint32 retSize, uint8* ram)
