@@ -8,8 +8,8 @@
 
 #include "Iop_Ioman.h"
 #include "IopBios.h"
-#include "../AppConfig.h"
-#include "../Log.h"
+#include "AppConfig.h"
+#include "Log.h"
 #include "../states/XmlStateFile.h"
 #include "ioman/PathDirectoryDevice.h"
 
@@ -41,8 +41,10 @@ using namespace Iop;
 #define PREF_IOP_FILEIO_STDLOGGING ("iop.fileio.stdlogging")
 
 #define FUNCTION_WRITE "Write"
+#define FUNCTION_CHSTAT "ChStat"
 #define FUNCTION_ADDDRV "AddDrv"
 #define FUNCTION_DELDRV "DelDrv"
+#define FUNCTION_RENAME "Rename"
 #define FUNCTION_MOUNT "Mount"
 #define FUNCTION_UMOUNT "Umount"
 #define FUNCTION_SEEK64 "Seek64"
@@ -204,12 +206,18 @@ std::string CIoman::GetFunctionName(unsigned int functionId) const
 	case 16:
 		return "getstat";
 		break;
+	case 17:
+		return FUNCTION_CHSTAT;
+		break;
 	case 20:
 		return FUNCTION_ADDDRV;
 		break;
 	case 21:
 		return FUNCTION_DELDRV;
 		break;
+	case 25:
+		return FUNCTION_RENAME;
+		break;
 	case 31:
 		return FUNCTION_DEVCTL;
 		break;
@@ -491,6 +499,13 @@ uint32 CIoman::GetStat(const char* path, Ioman::STAT* stat)
 	return -1;
 }
 
+int32 CIoman::ChStat(const char* path, Ioman::STAT* stat, uint32 statMask)
+{
+	CLog::GetInstance().Print(LOG_NAME, FUNCTION_CHSTAT "(path = '%s', stat = ptr, statMask = 0x%08X);\r\n",
+	                          path, statMask);
+	return GetStat(path, stat);
+}
+
 int32 CIoman::AddDrv(CMIPS& context)
 {
 	auto devicePtr = context.m_State.nGPR[CMIPS::A0].nV0;
@@ -502,14 +517,31 @@ int32 CIoman::AddDrv(CMIPS& context)
 	auto deviceName = device->namePtr ? reinterpret_cast<const char*>(m_ram + device->namePtr) : nullptr;
 	FRAMEWORK_MAYBE_UNUSED auto deviceDesc = device->descPtr ? reinterpret_cast<const char*>(m_ram + device->descPtr) : nullptr;
 	CLog::GetInstance().Print(LOG_NAME, "Requested registration of device '%s'.\r\n", deviceName);
-	//We only support "cdfs" & "dev9x" for now
-	if(!deviceName || (strcmp(deviceName, "cdfs") && strcmp(deviceName, "dev9x")))
+	if(!deviceName)
 	{
 		return -1;
 	}
-	m_userDevices.insert(std::make_pair(deviceName, devicePtr));
-	InvokeUserDeviceMethod(context, devicePtr, offsetof(Ioman::DEVICEOPS, initPtr), devicePtr);
-	return 0;
+
+	//We only allow "cdfs" & "dev9x" for now
+	static std::set<std::string> allowedDevices = {"cdfs", "dev9x"};
+	if(allowedDevices.count(deviceName))
+	{
+		m_userDevices.insert(std::make_pair(deviceName, devicePtr));
+		InvokeUserDeviceMethod(context, devicePtr, offsetof(Ioman::DEVICEOPS, initPtr), devicePtr);
+		return 0;
+	}
+
+	//We will report to some devices that registration succeeded, but we don't actually
+	//add it to our device list.
+	//"host" is needed for proper FF10 PSF initialization.
+	static std::set<std::string> silentDevices = {"cdrom_stm", "host"};
+	if(silentDevices.count(deviceName))
+	{
+		return 0;
+	}
+
+	//If we get here, we failed.
+	return -1;
 }
 
 uint32 CIoman::DelDrv(uint32 drvNamePtr)
@@ -519,6 +551,34 @@ uint32 CIoman::DelDrv(uint32 drvNamePtr)
 	return -1;
 }
 
+int32 CIoman::Rename(const char* srcPath, const char* dstPath)
+{
+	CLog::GetInstance().Print(LOG_NAME, FUNCTION_RENAME "(srcPath = '%s', dstPath = '%s');\r\n",
+	                          srcPath, dstPath);
+	int32 result = -1;
+	try
+	{
+		auto srcPathInfo = SplitPath(srcPath);
+		auto dstPathInfo = SplitPath(dstPath);
+		if(srcPathInfo.deviceName != dstPathInfo.deviceName)
+		{
+			throw std::runtime_error("Renaming files across devices not supported.");
+		}
+		auto deviceIterator = m_devices.find(srcPathInfo.deviceName);
+		if(deviceIterator == m_devices.end())
+		{
+			throw std::runtime_error("Device not found.");
+		}
+		deviceIterator->second->Rename(srcPathInfo.devicePath.c_str(), dstPathInfo.devicePath.c_str());
+	}
+	catch(const std::exception& except)
+	{
+		CLog::GetInstance().Warn(LOG_NAME, "%s: Error occured while trying to rename file '%s' to '%s'\r\n",
+		                         __FUNCTION__, srcPath, dstPath, except.what());
+	}
+	return result;
+}
+
 int32 CIoman::DevCtlVirtual(CMIPS& context)
 {
 	uint32 deviceNamePtr = context.m_State.nGPR[CMIPS::A0].nV0;
@@ -1019,6 +1079,12 @@ void CIoman::Invoke(CMIPS& context, unsigned int functionId)
 		    reinterpret_cast<char*>(&m_ram[context.m_State.nGPR[CMIPS::A0].nV[0]]),
 		    reinterpret_cast<Ioman::STAT*>(&m_ram[context.m_State.nGPR[CMIPS::A1].nV[0]])));
 		break;
+	case 17:
+		context.m_State.nGPR[CMIPS::V0].nD0 = ChStat(
+		    reinterpret_cast<const char*>(&m_ram[context.m_State.nGPR[CMIPS::A0].nV[0]]),
+		    reinterpret_cast<Ioman::STAT*>(&m_ram[context.m_State.nGPR[CMIPS::A1].nV[0]]),
+		    context.m_State.nGPR[CMIPS::A2].nV[0]);
+		break;
 	case 20:
 		context.m_State.nGPR[CMIPS::V0].nD0 = static_cast<int32>(AddDrv(context));
 		break;
@@ -1026,6 +1092,11 @@ void CIoman::Invoke(CMIPS& context, unsigned int functionId)
 		context.m_State.nGPR[CMIPS::V0].nD0 = static_cast<int32>(DelDrv(
 		    context.m_State.nGPR[CMIPS::A0].nV0));
 		break;
+	case 25:
+		context.m_State.nGPR[CMIPS::V0].nD0 = Rename(
+		    reinterpret_cast<const char*>(&m_ram[context.m_State.nGPR[CMIPS::A0].nV[0]]),
+		    reinterpret_cast<const char*>(&m_ram[context.m_State.nGPR[CMIPS::A1].nV[0]]));
+		break;
 	case 31:
 		context.m_State.nGPR[CMIPS::V0].nD0 = DevCtlVirtual(context);
 		break;
