@@ -4,7 +4,7 @@
 #include <climits>
 #include <algorithm>
 #include "string_format.h"
-#include "../Log.h"
+#include "Log.h"
 #include "../states/RegisterStateCollectionFile.h"
 #include "../states/RegisterStateUtils.h"
 #include "../states/RegisterStateFile.h"
@@ -489,6 +489,10 @@ void CSpuBase::SendKeyOn(uint32 channels)
 		{
 			channel.status = KEY_ON;
 			channel.repeatSet = false;
+			channel.current = channel.address;
+			//Simulate one tick of ADSR in case game tries to read envelope value quickly
+			//(ie.: Before 'Render' is called and while the voice is still in KEY_ON state)
+			channel.adsrVolume = GetAdsrDelta((channel.adsrLevel.attackRate ^ 0x7F) - 0x10);
 		}
 	}
 }
@@ -977,8 +981,8 @@ void CSpuBase::UpdateReverb(int16 reverbSample[2], int16* samples)
 	{
 		//IIR_INPUT_A0 = buffer[IIR_SRC_A0] * IIR_COEF + INPUT_SAMPLE_L * IN_COEF_L;
 		//IIR_INPUT_A1 = buffer[IIR_SRC_A1] * IIR_COEF + INPUT_SAMPLE_R * IN_COEF_R;
-		//IIR_INPUT_B0 = buffer[IIR_SRC_B0] * IIR_COEF + INPUT_SAMPLE_L * IN_COEF_L;
-		//IIR_INPUT_B1 = buffer[IIR_SRC_B1] * IIR_COEF + INPUT_SAMPLE_R * IN_COEF_R;
+		//IIR_INPUT_B0 = buffer[IIR_SRC_B1] * IIR_COEF + INPUT_SAMPLE_L * IN_COEF_L;
+		//IIR_INPUT_B1 = buffer[IIR_SRC_B0] * IIR_COEF + INPUT_SAMPLE_R * IN_COEF_R;
 
 		float input_sample_l = static_cast<float>(reverbSample[0]) * 0.5f;
 		float input_sample_r = static_cast<float>(reverbSample[1]) * 0.5f;
@@ -987,10 +991,10 @@ void CSpuBase::UpdateReverb(int16 reverbSample[2], int16* samples)
 		float in_coef_l = GetReverbCoef(IN_COEF_L);
 		float in_coef_r = GetReverbCoef(IN_COEF_R);
 
-		float iir_input_a0 = GetReverbSample(GetReverbOffset(ACC_SRC_A0)) * irr_coef + input_sample_l * in_coef_l;
-		float iir_input_a1 = GetReverbSample(GetReverbOffset(ACC_SRC_A1)) * irr_coef + input_sample_r * in_coef_r;
-		float iir_input_b0 = GetReverbSample(GetReverbOffset(ACC_SRC_B0)) * irr_coef + input_sample_l * in_coef_l;
-		float iir_input_b1 = GetReverbSample(GetReverbOffset(ACC_SRC_B1)) * irr_coef + input_sample_r * in_coef_r;
+		float iir_input_a0 = GetReverbSample(GetReverbOffset(IIR_SRC_A0)) * irr_coef + input_sample_l * in_coef_l;
+		float iir_input_a1 = GetReverbSample(GetReverbOffset(IIR_SRC_A1)) * irr_coef + input_sample_r * in_coef_r;
+		float iir_input_b0 = GetReverbSample(GetReverbOffset(IIR_SRC_B1)) * irr_coef + input_sample_l * in_coef_l;
+		float iir_input_b1 = GetReverbSample(GetReverbOffset(IIR_SRC_B0)) * irr_coef + input_sample_r * in_coef_r;
 
 		//IIR_A0 = IIR_INPUT_A0 * IIR_ALPHA + buffer[IIR_DEST_A0] * (1.0 - IIR_ALPHA);
 		//IIR_A1 = IIR_INPUT_A1 * IIR_ALPHA + buffer[IIR_DEST_A1] * (1.0 - IIR_ALPHA);
