@@ -10,12 +10,15 @@ using namespace Jitter;
 #include "Jitter_CodeGen_x86_Mul.h"
 #include "Jitter_CodeGen_x86_Div.h"
 
-const LITERAL128 CCodeGen_x86::g_makeSzShufflePattern = { 0x00020406080A0C0E, 0x8080808080808080 };
-const LITERAL128 CCodeGen_x86::g_fpClampMask1 = { 0x7F7FFFFF, 0x7F7FFFFF, 0x7F7FFFFF, 0x7F7FFFFF };
-const LITERAL128 CCodeGen_x86::g_fpClampMask2 = { 0xFF7FFFFF, 0xFF7FFFFF, 0xFF7FFFFF, 0xFF7FFFFF };
+const LITERAL128 CCodeGen_x86::g_makeClipShufflePattern = {0x80800C040A020800, 0x8080808080808080};
+const LITERAL128 CCodeGen_x86::g_makeSzShufflePattern = {0x00020406080A0C0E, 0x8080808080808080};
+const LITERAL128 CCodeGen_x86::g_fpCstOne = {0x3F800000, 0x3F800000, 0x3F800000, 0x3F800000};
+const LITERAL128 CCodeGen_x86::g_fpClampMask1 = {0x7F7FFFFF, 0x7F7FFFFF, 0x7F7FFFFF, 0x7F7FFFFF};
+const LITERAL128 CCodeGen_x86::g_fpClampMask2 = {0xFF7FFFFF, 0xFF7FFFFF, 0xFF7FFFFF, 0xFF7FFFFF};
 
 CX86Assembler::REGISTER CCodeGen_x86::g_baseRegister = CX86Assembler::rBP;
 
+// clang-format off
 CCodeGen_x86::CONSTMATCHER CCodeGen_x86::g_constMatchers[] = 
 { 
 	{ OP_LABEL, MATCH_NIL, MATCH_NIL, MATCH_NIL, MATCH_NIL, &CCodeGen_x86::MarkLabel  },
@@ -56,35 +59,24 @@ CCodeGen_x86::CONSTMATCHER CCodeGen_x86::g_constMatchers[] =
 	{ OP_CONDJMP, MATCH_NIL, MATCH_MEMORY,   MATCH_MEMORY,   MATCH_NIL, &CCodeGen_x86::Emit_CondJmp_MemMem },
 	{ OP_CONDJMP, MATCH_NIL, MATCH_MEMORY,   MATCH_CONSTANT, MATCH_NIL, &CCodeGen_x86::Emit_CondJmp_MemCst },
 
-	{ OP_DIV, MATCH_TEMPORARY64, MATCH_REGISTER, MATCH_REGISTER, MATCH_NIL, &CCodeGen_x86::Emit_DivTmp64RegReg<false> },
-	{ OP_DIV, MATCH_TEMPORARY64, MATCH_REGISTER, MATCH_MEMORY,   MATCH_NIL, &CCodeGen_x86::Emit_DivTmp64RegMem<false> },
-	{ OP_DIV, MATCH_TEMPORARY64, MATCH_REGISTER, MATCH_CONSTANT, MATCH_NIL, &CCodeGen_x86::Emit_DivTmp64RegCst<false> },
-	{ OP_DIV, MATCH_TEMPORARY64, MATCH_MEMORY,   MATCH_REGISTER, MATCH_NIL, &CCodeGen_x86::Emit_DivTmp64MemReg<false> },
-	{ OP_DIV, MATCH_TEMPORARY64, MATCH_MEMORY,   MATCH_MEMORY,   MATCH_NIL, &CCodeGen_x86::Emit_DivTmp64MemMem<false> },
-	{ OP_DIV, MATCH_TEMPORARY64, MATCH_MEMORY,   MATCH_CONSTANT, MATCH_NIL, &CCodeGen_x86::Emit_DivTmp64MemCst<false> },
-	{ OP_DIV, MATCH_TEMPORARY64, MATCH_CONSTANT, MATCH_REGISTER, MATCH_NIL, &CCodeGen_x86::Emit_DivTmp64CstReg<false> },
-	{ OP_DIV, MATCH_TEMPORARY64, MATCH_CONSTANT, MATCH_MEMORY,   MATCH_NIL, &CCodeGen_x86::Emit_DivTmp64CstMem<false> },
+	{ OP_CMPSELECT_P1, MATCH_NIL,      MATCH_ANY, MATCH_VARIABLE, MATCH_NIL, &CCodeGen_x86::Emit_CmpSelectP1_AnyVar    },
+	{ OP_CMPSELECT_P2, MATCH_VARIABLE, MATCH_ANY, MATCH_ANY,      MATCH_NIL, &CCodeGen_x86::Emit_CmpSelectP2_VarAnyAny },
 
-	{ OP_DIVS, MATCH_TEMPORARY64, MATCH_REGISTER, MATCH_REGISTER, MATCH_NIL, &CCodeGen_x86::Emit_DivTmp64RegReg<true> },
-	{ OP_DIVS, MATCH_TEMPORARY64, MATCH_REGISTER, MATCH_MEMORY,   MATCH_NIL, &CCodeGen_x86::Emit_DivTmp64RegMem<true> },
-	{ OP_DIVS, MATCH_TEMPORARY64, MATCH_REGISTER, MATCH_CONSTANT, MATCH_NIL, &CCodeGen_x86::Emit_DivTmp64RegCst<true> },
-	{ OP_DIVS, MATCH_TEMPORARY64, MATCH_MEMORY,   MATCH_REGISTER, MATCH_NIL, &CCodeGen_x86::Emit_DivTmp64MemReg<true> },
-	{ OP_DIVS, MATCH_TEMPORARY64, MATCH_MEMORY,   MATCH_MEMORY,   MATCH_NIL, &CCodeGen_x86::Emit_DivTmp64MemMem<true> },
-	{ OP_DIVS, MATCH_TEMPORARY64, MATCH_MEMORY,   MATCH_CONSTANT, MATCH_NIL, &CCodeGen_x86::Emit_DivTmp64MemCst<true> },
-	{ OP_DIVS, MATCH_TEMPORARY64, MATCH_CONSTANT, MATCH_REGISTER, MATCH_NIL, &CCodeGen_x86::Emit_DivTmp64CstReg<true> },
-	{ OP_DIVS, MATCH_TEMPORARY64, MATCH_CONSTANT, MATCH_MEMORY,   MATCH_NIL, &CCodeGen_x86::Emit_DivTmp64CstMem<true> },
+	{ OP_SELECT, MATCH_VARIABLE, MATCH_VARIABLE, MATCH_ANY, MATCH_ANY, &CCodeGen_x86::Emit_Select_VarVarAnyAny },
 
-	{ OP_MUL, MATCH_TEMPORARY64, MATCH_REGISTER, MATCH_REGISTER, MATCH_NIL, &CCodeGen_x86::Emit_MulTmp64RegReg<false> },
-	{ OP_MUL, MATCH_TEMPORARY64, MATCH_REGISTER, MATCH_MEMORY,   MATCH_NIL, &CCodeGen_x86::Emit_MulTmp64RegMem<false> },
-	{ OP_MUL, MATCH_TEMPORARY64, MATCH_REGISTER, MATCH_CONSTANT, MATCH_NIL, &CCodeGen_x86::Emit_MulTmp64RegCst<false> },
-	{ OP_MUL, MATCH_TEMPORARY64, MATCH_MEMORY,   MATCH_MEMORY,   MATCH_NIL, &CCodeGen_x86::Emit_MulTmp64MemMem<false> },
-	{ OP_MUL, MATCH_TEMPORARY64, MATCH_MEMORY,   MATCH_CONSTANT, MATCH_NIL, &CCodeGen_x86::Emit_MulTmp64MemCst<false> },
+	{ OP_DIV, MATCH_MEMORY64, MATCH_VARIABLE, MATCH_VARIABLE, MATCH_NIL, &CCodeGen_x86::Emit_DivMem64VarVar<false> },
+	{ OP_DIV, MATCH_MEMORY64, MATCH_VARIABLE, MATCH_CONSTANT, MATCH_NIL, &CCodeGen_x86::Emit_DivMem64VarCst<false> },
+	{ OP_DIV, MATCH_MEMORY64, MATCH_CONSTANT, MATCH_VARIABLE, MATCH_NIL, &CCodeGen_x86::Emit_DivMem64CstVar<false> },
 
-	{ OP_MULS, MATCH_TEMPORARY64, MATCH_REGISTER, MATCH_REGISTER, MATCH_NIL, &CCodeGen_x86::Emit_MulTmp64RegReg<true> },
-	{ OP_MULS, MATCH_TEMPORARY64, MATCH_REGISTER, MATCH_MEMORY,   MATCH_NIL, &CCodeGen_x86::Emit_MulTmp64RegMem<true> },
-	{ OP_MULS, MATCH_TEMPORARY64, MATCH_REGISTER, MATCH_CONSTANT, MATCH_NIL, &CCodeGen_x86::Emit_MulTmp64RegCst<true> },
-	{ OP_MULS, MATCH_TEMPORARY64, MATCH_MEMORY,   MATCH_MEMORY,   MATCH_NIL, &CCodeGen_x86::Emit_MulTmp64MemMem<true> },
-	{ OP_MULS, MATCH_TEMPORARY64, MATCH_MEMORY,   MATCH_CONSTANT, MATCH_NIL, &CCodeGen_x86::Emit_MulTmp64MemCst<true> },
+	{ OP_DIVS, MATCH_MEMORY64, MATCH_VARIABLE, MATCH_VARIABLE, MATCH_NIL, &CCodeGen_x86::Emit_DivMem64VarVar<true> },
+	{ OP_DIVS, MATCH_MEMORY64, MATCH_VARIABLE, MATCH_CONSTANT, MATCH_NIL, &CCodeGen_x86::Emit_DivMem64VarCst<true> },
+	{ OP_DIVS, MATCH_MEMORY64, MATCH_CONSTANT, MATCH_VARIABLE, MATCH_NIL, &CCodeGen_x86::Emit_DivMem64CstVar<true> },
+
+	{ OP_MUL, MATCH_MEMORY64, MATCH_VARIABLE, MATCH_VARIABLE, MATCH_NIL, &CCodeGen_x86::Emit_MulMem64VarVar<false> },
+	{ OP_MUL, MATCH_MEMORY64, MATCH_VARIABLE, MATCH_CONSTANT, MATCH_NIL, &CCodeGen_x86::Emit_MulMem64VarCst<false> },
+
+	{ OP_MULS, MATCH_MEMORY64, MATCH_VARIABLE, MATCH_VARIABLE, MATCH_NIL, &CCodeGen_x86::Emit_MulMem64VarVar<true> },
+	{ OP_MULS, MATCH_MEMORY64, MATCH_VARIABLE, MATCH_CONSTANT, MATCH_NIL, &CCodeGen_x86::Emit_MulMem64VarCst<true> },
 
 	{ OP_MERGETO64, MATCH_MEMORY64, MATCH_REGISTER, MATCH_REGISTER, MATCH_NIL, &CCodeGen_x86::Emit_MergeTo64_Mem64RegReg },
 	{ OP_MERGETO64, MATCH_MEMORY64, MATCH_REGISTER, MATCH_MEMORY,   MATCH_NIL, &CCodeGen_x86::Emit_MergeTo64_Mem64RegMem },
@@ -94,11 +86,8 @@ CCodeGen_x86::CONSTMATCHER CCodeGen_x86::g_constMatchers[] =
 	{ OP_MERGETO64, MATCH_MEMORY64, MATCH_CONSTANT, MATCH_REGISTER, MATCH_NIL, &CCodeGen_x86::Emit_MergeTo64_Mem64CstReg },
 	{ OP_MERGETO64, MATCH_MEMORY64, MATCH_CONSTANT, MATCH_MEMORY,   MATCH_NIL, &CCodeGen_x86::Emit_MergeTo64_Mem64CstMem },
 
-	{ OP_EXTLOW64, MATCH_REGISTER, MATCH_TEMPORARY64, MATCH_NIL, MATCH_NIL, &CCodeGen_x86::Emit_ExtLow64RegTmp64 },
-	{ OP_EXTLOW64, MATCH_MEMORY,   MATCH_TEMPORARY64, MATCH_NIL, MATCH_NIL, &CCodeGen_x86::Emit_ExtLow64MemTmp64 },
-	
-	{ OP_EXTHIGH64, MATCH_REGISTER, MATCH_TEMPORARY64, MATCH_NIL, MATCH_NIL, &CCodeGen_x86::Emit_ExtHigh64RegTmp64 },
-	{ OP_EXTHIGH64, MATCH_MEMORY,   MATCH_TEMPORARY64, MATCH_NIL, MATCH_NIL, &CCodeGen_x86::Emit_ExtHigh64MemTmp64 },
+	{ OP_EXTLOW64,  MATCH_VARIABLE, MATCH_MEMORY64, MATCH_NIL, MATCH_NIL, &CCodeGen_x86::Emit_ExtLow64VarMem64 },
+	{ OP_EXTHIGH64, MATCH_VARIABLE, MATCH_MEMORY64, MATCH_NIL, MATCH_NIL, &CCodeGen_x86::Emit_ExtHigh64VarMem64 },
 
 	{ OP_LOADFROMREF, MATCH_VARIABLE,    MATCH_VAR_REF, MATCH_NIL,   MATCH_NIL, &CCodeGen_x86::Emit_LoadFromRef_VarVar    },
 	{ OP_LOADFROMREF, MATCH_VARIABLE,    MATCH_VAR_REF, MATCH_ANY32, MATCH_NIL, &CCodeGen_x86::Emit_LoadFromRef_VarVarAny },
@@ -124,6 +113,7 @@ CCodeGen_x86::CONSTMATCHER CCodeGen_x86::g_constMatchers[] =
 
 	{ OP_MOV, MATCH_NIL, MATCH_NIL, MATCH_NIL, MATCH_NIL, nullptr },
 };
+// clang-format on
 
 CCodeGen_x86::CCodeGen_x86(CX86CpuFeatures cpuFeatures)
     : m_cpuFeatures(cpuFeatures)
@@ -138,36 +128,34 @@ CCodeGen_x86::CCodeGen_x86(CX86CpuFeatures cpuFeatures)
 
 		if(cpuFeatures.hasAvx2)
 		{
-			InsertMatchers(g_mdAvx2ExpandConstMatchers);
+			InsertMatchers(g_mdAvx2ConstMatchers);
 		}
 		else
 		{
-			InsertMatchers(g_mdAvxExpandConstMatchers);
+			InsertMatchers(g_mdNoAvx2ConstMatchers);
 		}
 	}
 	else
 	{
 		InsertMatchers(g_fpuSseConstMatchers);
-		InsertMatchers(g_mdConstMatchers);
+		InsertMatchers(g_mdSseConstMatchers);
 
 		if(cpuFeatures.hasSsse3)
 		{
-			InsertMatchers(g_mdFpFlagSsse3ConstMatchers);
+			InsertMatchers(g_mdSsse3ConstMatchers);
 		}
 		else
 		{
-			InsertMatchers(g_mdFpFlagConstMatchers);
+			InsertMatchers(g_mdNoSsse3ConstMatchers);
 		}
 
 		if(cpuFeatures.hasSse41)
 		{
-			InsertMatchers(g_mdMinMaxWSse41ConstMatchers);
-			InsertMatchers(g_mdMovMaskedSse41ConstMatchers);
+			InsertMatchers(g_mdSse41ConstMatchers);
 		}
 		else
 		{
-			InsertMatchers(g_mdMinMaxWConstMatchers);
-			InsertMatchers(g_mdMovMaskedConstMatchers);
+			InsertMatchers(g_mdNoSse41ConstMatchers);
 		}
 	}
 }
@@ -179,7 +167,7 @@ void CCodeGen_x86::GenerateCode(const StatementList& statements, unsigned int st
 	assert(m_labels.empty());
 
 	m_registerUsage = GetRegisterUsage(statements);
-	
+
 	//Align stacksize
 	stackSize = (stackSize + 0xF) & ~0xF;
 	m_stackLevel = 0;
@@ -238,12 +226,12 @@ void CCodeGen_x86::InsertMatchers(const CONSTMATCHER* constMatchers)
 	for(auto* constMatcher = constMatchers; constMatcher->emitter != nullptr; constMatcher++)
 	{
 		MATCHER matcher;
-		matcher.op       = constMatcher->op;
-		matcher.dstType  = constMatcher->dstType;
+		matcher.op = constMatcher->op;
+		matcher.dstType = constMatcher->dstType;
 		matcher.src1Type = constMatcher->src1Type;
 		matcher.src2Type = constMatcher->src2Type;
 		matcher.src3Type = constMatcher->src3Type;
-		matcher.emitter  = std::bind(constMatcher->emitter, this, std::placeholders::_1);
+		matcher.emitter = std::bind(constMatcher->emitter, this, std::placeholders::_1);
 		m_matchers.insert(MatcherMapType::value_type(matcher.op, matcher));
 	}
 }
@@ -268,6 +256,11 @@ bool CCodeGen_x86::SupportsExternalJumps() const
 	return true;
 }
 
+bool CCodeGen_x86::SupportsCmpSelect() const
+{
+	return true;
+}
+
 CX86Assembler::LABEL CCodeGen_x86::GetLabel(uint32 blockId)
 {
 	CX86Assembler::LABEL result;
@@ -385,7 +378,7 @@ CX86Assembler::CAddress CCodeGen_x86::MakeVariableReferenceSymbolAddress(CSymbol
 }
 
 CX86Assembler::CAddress CCodeGen_x86::MakeRefBaseScaleSymbolAddress(CSymbol* baseSymbol, CX86Assembler::REGISTER baseRegister,
-	CSymbol* indexSymbol, CX86Assembler::REGISTER indexRegister, uint8 scale)
+                                                                    CSymbol* indexSymbol, CX86Assembler::REGISTER indexRegister, uint8 scale)
 {
 	baseRegister = PrepareRefSymbolRegisterUse(baseSymbol, baseRegister);
 	if(indexSymbol->IsConstant())
@@ -498,7 +491,6 @@ void CCodeGen_x86::MarkLabel(const STATEMENT& statement)
 
 void CCodeGen_x86::Emit_Nop(const STATEMENT& statement)
 {
-	
 }
 
 void CCodeGen_x86::Emit_Break(const STATEMENT& statement)
@@ -717,11 +709,11 @@ void CCodeGen_x86::Emit_MergeTo64_Mem64MemReg(const STATEMENT& statement)
 	auto dst = statement.dst->GetSymbol().get();
 	auto src1 = statement.src1->GetSymbol().get();
 	auto src2 = statement.src2->GetSymbol().get();
-	
+
 	assert(src2->m_type == SYM_REGISTER);
-	
+
 	m_assembler.MovEd(CX86Assembler::rAX, MakeMemorySymbolAddress(src1));
-	
+
 	m_assembler.MovGd(MakeMemory64SymbolLoAddress(dst), CX86Assembler::rAX);
 	m_assembler.MovGd(MakeMemory64SymbolHiAddress(dst), m_registers[src2->m_valueLow]);
 }
@@ -769,48 +761,24 @@ void CCodeGen_x86::Emit_MergeTo64_Mem64CstMem(const STATEMENT& statement)
 	m_assembler.MovGd(MakeMemory64SymbolHiAddress(dst), CX86Assembler::rDX);
 }
 
-void CCodeGen_x86::Emit_ExtLow64RegTmp64(const STATEMENT& statement)
-{
-	CSymbol* dst = statement.dst->GetSymbol().get();
-	CSymbol* src1 = statement.src1->GetSymbol().get();
-
-	assert(dst->m_type  == SYM_REGISTER);
-	assert(src1->m_type == SYM_TEMPORARY64);
-
-	m_assembler.MovEd(m_registers[dst->m_valueLow], CX86Assembler::MakeIndRegOffAddress(CX86Assembler::rSP, src1->m_stackLocation + m_stackLevel + 0));
-}
-
-void CCodeGen_x86::Emit_ExtLow64MemTmp64(const STATEMENT& statement)
+void CCodeGen_x86::Emit_ExtLow64VarMem64(const STATEMENT& statement)
 {
 	auto dst = statement.dst->GetSymbol().get();
 	auto src1 = statement.src1->GetSymbol().get();
 
-	assert(src1->m_type == SYM_TEMPORARY64);
-
-	m_assembler.MovEd(CX86Assembler::rAX, CX86Assembler::MakeIndRegOffAddress(CX86Assembler::rSP, src1->m_stackLocation + m_stackLevel + 0));
-	m_assembler.MovGd(MakeMemorySymbolAddress(dst), CX86Assembler::rAX);
+	auto dstReg = PrepareSymbolRegisterDef(dst, CX86Assembler::rAX);
+	m_assembler.MovEd(dstReg, MakeMemory64SymbolLoAddress(src1));
+	CommitSymbolRegister(dst, dstReg);
 }
 
-void CCodeGen_x86::Emit_ExtHigh64RegTmp64(const STATEMENT& statement)
-{
-	CSymbol* dst = statement.dst->GetSymbol().get();
-	CSymbol* src1 = statement.src1->GetSymbol().get();
-
-	assert(dst->m_type  == SYM_REGISTER);
-	assert(src1->m_type == SYM_TEMPORARY64);
-
-	m_assembler.MovEd(m_registers[dst->m_valueLow], CX86Assembler::MakeIndRegOffAddress(CX86Assembler::rSP, src1->m_stackLocation + m_stackLevel + 4));
-}
-
-void CCodeGen_x86::Emit_ExtHigh64MemTmp64(const STATEMENT& statement)
+void CCodeGen_x86::Emit_ExtHigh64VarMem64(const STATEMENT& statement)
 {
 	auto dst = statement.dst->GetSymbol().get();
 	auto src1 = statement.src1->GetSymbol().get();
 
-	assert(src1->m_type == SYM_TEMPORARY64);
-
-	m_assembler.MovEd(CX86Assembler::rAX, CX86Assembler::MakeIndRegOffAddress(CX86Assembler::rSP, src1->m_stackLocation + m_stackLevel + 4));
-	m_assembler.MovGd(MakeMemorySymbolAddress(dst), CX86Assembler::rAX);
+	auto dstReg = PrepareSymbolRegisterDef(dst, CX86Assembler::rAX);
+	m_assembler.MovEd(dstReg, MakeMemory64SymbolHiAddress(src1));
+	CommitSymbolRegister(dst, dstReg);
 }
 
 void CCodeGen_x86::Emit_LoadFromRef_VarVar(const STATEMENT& statement)
@@ -919,7 +887,7 @@ void CCodeGen_x86::Emit_StoreAtRef_VarAnyVar(const STATEMENT& statement)
 	auto src2 = statement.src2->GetSymbol().get();
 	auto src3 = statement.src3->GetSymbol().get();
 	uint8 scale = static_cast<uint8>(statement.jmpCondition);
-	
+
 	auto valueReg = PrepareSymbolRegisterUse(src3, CX86Assembler::rDX);
 	m_assembler.MovGd(MakeRefBaseScaleSymbolAddress(src1, CX86Assembler::rAX, src2, CX86Assembler::rCX, scale), valueReg);
 }
@@ -987,7 +955,7 @@ void CCodeGen_x86::Emit_Store16AtRef_VarAnyVar(const STATEMENT& statement)
 	auto src2 = statement.src2->GetSymbol().get();
 	auto src3 = statement.src3->GetSymbol().get();
 	FRAMEWORK_MAYBE_UNUSED uint8 scale = static_cast<uint8>(statement.jmpCondition);
-	
+
 	assert(scale == 1);
 
 	auto valueReg = PrepareSymbolRegisterUse(src3, CX86Assembler::rDX);
@@ -1020,6 +988,9 @@ void CCodeGen_x86::Cmp_GetFlag(const CX86Assembler::CAddress& dst, CONDITION fla
 	case CONDITION_GT:
 		m_assembler.SetgEb(dst);
 		break;
+	case CONDITION_GE:
+		m_assembler.SetgeEb(dst);
+		break;
 	case CONDITION_EQ:
 		m_assembler.SeteEb(dst);
 		break;
@@ -1029,9 +1000,15 @@ void CCodeGen_x86::Cmp_GetFlag(const CX86Assembler::CAddress& dst, CONDITION fla
 	case CONDITION_BL:
 		m_assembler.SetbEb(dst);
 		break;
+	case CONDITION_BE:
+		m_assembler.SetbeEb(dst);
+		break;
 	case CONDITION_AB:
 		m_assembler.SetaEb(dst);
 		break;
+	case CONDITION_AE:
+		m_assembler.SetaeEb(dst);
+		break;
 	default:
 		assert(0);
 		break;
@@ -1142,7 +1119,7 @@ void CCodeGen_x86::Emit_CondJmp_MemCst(const STATEMENT& statement)
 	assert(src2->m_type == SYM_CONSTANT);
 
 	m_assembler.MovEd(CX86Assembler::rAX, MakeMemorySymbolAddress(src1));
-	
+
 	if((src2->m_valueLow == 0) && (statement.jmpCondition == CONDITION_NE || statement.jmpCondition == CONDITION_EQ))
 	{
 		m_assembler.TestEd(CX86Assembler::rAX, CX86Assembler::MakeRegisterAddress(CX86Assembler::rAX));
@@ -1155,6 +1132,113 @@ void CCodeGen_x86::Emit_CondJmp_MemCst(const STATEMENT& statement)
 	CondJmp_JumpTo(GetLabel(statement.jmpBlock), statement.jmpCondition);
 }
 
+void CCodeGen_x86::Select_Generic(Jitter::CONDITION cond, CSymbol* dstSymbol, CSymbol* trueSymbol, CSymbol* falseSymbol)
+{
+	auto dstReg = PrepareSymbolRegisterDef(dstSymbol, CX86Assembler::rAX);
+
+	auto emitMovCc = [&](CONDITION cond, CX86Assembler::REGISTER dstReg, const CX86Assembler::CAddress& srcAddress) {
+		switch(cond)
+		{
+		case CONDITION_EQ:
+			m_assembler.CmoveEd(dstReg, srcAddress);
+			break;
+		case CONDITION_NE:
+			m_assembler.CmovneEd(dstReg, srcAddress);
+			break;
+		case CONDITION_LE:
+			m_assembler.CmovleEd(dstReg, srcAddress);
+			break;
+		case CONDITION_GT:
+			m_assembler.CmovgEd(dstReg, srcAddress);
+			break;
+		default:
+			assert(false);
+			break;
+		}
+	};
+
+	auto makeValueSymbolAddress =
+	    [&](CSymbol* valueSymbol) {
+		    if(valueSymbol->IsConstant())
+		    {
+			    auto tmpReg = CX86Assembler::rCX;
+			    m_assembler.MovId(tmpReg, valueSymbol->m_valueLow);
+			    return CX86Assembler::MakeRegisterAddress(tmpReg);
+		    }
+		    else
+		    {
+			    return MakeVariableSymbolAddress(valueSymbol);
+		    }
+	    };
+
+	//The order of tests here is important
+	if(trueSymbol->IsConstant() && falseSymbol->IsConstant())
+	{
+		assert(trueSymbol->m_valueLow != falseSymbol->m_valueLow);
+		auto tmpReg = CX86Assembler::rCX;
+		m_assembler.MovId(dstReg, trueSymbol->m_valueLow);
+		m_assembler.MovId(tmpReg, falseSymbol->m_valueLow);
+		emitMovCc(NegateCondition(cond), dstReg, CX86Assembler::MakeRegisterAddress(tmpReg));
+	}
+	else if(dstSymbol->IsRegister() && dstSymbol->Equals(trueSymbol))
+	{
+		//dst is already true, move false if !cond
+		emitMovCc(NegateCondition(cond), dstReg, makeValueSymbolAddress(falseSymbol));
+	}
+	else if(dstSymbol->IsRegister() && dstSymbol->Equals(falseSymbol))
+	{
+		//dst is already false, move true if cond
+		emitMovCc(cond, dstReg, makeValueSymbolAddress(trueSymbol));
+	}
+	else if(trueSymbol->IsConstant())
+	{
+		m_assembler.MovId(dstReg, trueSymbol->m_valueLow);
+		emitMovCc(NegateCondition(cond), dstReg, MakeVariableSymbolAddress(falseSymbol));
+	}
+	else if(falseSymbol->IsConstant())
+	{
+		m_assembler.MovId(dstReg, falseSymbol->m_valueLow);
+		emitMovCc(cond, dstReg, MakeVariableSymbolAddress(trueSymbol));
+	}
+	else
+	{
+		m_assembler.MovEd(dstReg, MakeVariableSymbolAddress(trueSymbol));
+		emitMovCc(NegateCondition(cond), dstReg, MakeVariableSymbolAddress(falseSymbol));
+	}
+
+	CommitSymbolRegister(dstSymbol, dstReg);
+}
+
+void CCodeGen_x86::Emit_Select_VarVarAnyAny(const STATEMENT& statement)
+{
+	auto dst = statement.dst->GetSymbol().get();
+	auto src1 = statement.src1->GetSymbol().get();
+	auto src2 = statement.src2->GetSymbol().get();
+	auto src3 = statement.src3->GetSymbol().get();
+
+	m_assembler.CmpId(MakeVariableSymbolAddress(src1), 0);
+	Select_Generic(CONDITION_NE, dst, src2, src3);
+}
+
+void CCodeGen_x86::Emit_CmpSelectP1_AnyVar(const STATEMENT& statement)
+{
+	auto src1 = statement.src1->GetSymbol().get();
+	auto src2 = statement.src2->GetSymbol().get();
+
+	auto src1Reg = PrepareSymbolRegisterUse(src1, CX86Assembler::rDX);
+
+	m_assembler.CmpEd(src1Reg, MakeVariableSymbolAddress(src2));
+}
+
+void CCodeGen_x86::Emit_CmpSelectP2_VarAnyAny(const STATEMENT& statement)
+{
+	auto dst = statement.dst->GetSymbol().get();
+	auto src1 = statement.src1->GetSymbol().get();
+	auto src2 = statement.src2->GetSymbol().get();
+
+	Select_Generic(statement.jmpCondition, dst, src1, src2);
+}
+
 CX86Assembler::REGISTER CCodeGen_x86::PrepareSymbolRegisterDef(CSymbol* symbol, CX86Assembler::REGISTER preferedRegister)
 {
 	switch(symbol->m_type)
@@ -1201,19 +1285,19 @@ CX86Assembler::BYTEREGISTER CCodeGen_x86::PrepareSymbolByteRegisterUse(CSymbol*
 	switch(symbol->m_type)
 	{
 	case SYM_REGISTER:
+	{
+		auto srcRegister = m_registers[symbol->m_valueLow];
+		if(CX86Assembler::HasByteRegister(srcRegister))
 		{
-			auto srcRegister = m_registers[symbol->m_valueLow];
-			if(CX86Assembler::HasByteRegister(srcRegister))
-			{
-				return CX86Assembler::GetByteRegister(srcRegister);
-			}
-			else
-			{
-				m_assembler.MovEd(preferedRegister, CX86Assembler::MakeRegisterAddress(srcRegister));
-				return preferedByteRegister;
-			}
+			return CX86Assembler::GetByteRegister(srcRegister);
 		}
-		break;
+		else
+		{
+			m_assembler.MovEd(preferedRegister, CX86Assembler::MakeRegisterAddress(srcRegister));
+			return preferedByteRegister;
+		}
+	}
+	break;
 	case SYM_TEMPORARY:
 	case SYM_RELATIVE:
 		m_assembler.MovEd(preferedRegister, MakeMemorySymbolAddress(symbol));
@@ -1246,6 +1330,23 @@ void CCodeGen_x86::CommitSymbolRegister(CSymbol* symbol, CX86Assembler::REGISTER
 	}
 }
 
+CX86Assembler::XMMREGISTER CCodeGen_x86::PrepareSymbolRegisterDefFp32(CSymbol* symbol, CX86Assembler::XMMREGISTER preferedRegister)
+{
+	switch(symbol->m_type)
+	{
+	case SYM_FP_REGISTER32:
+		return m_mdRegisters[symbol->m_valueLow];
+		break;
+	case SYM_FP_TEMPORARY32:
+	case SYM_FP_RELATIVE32:
+		return preferedRegister;
+		break;
+	default:
+		throw std::runtime_error("Invalid symbol type.");
+		break;
+	}
+}
+
 CX86Assembler::XMMREGISTER CCodeGen_x86::PrepareSymbolRegisterDefMd(CSymbol* symbol, CX86Assembler::XMMREGISTER preferedRegister)
 {
 	switch(symbol->m_type)
