@@ -38,6 +38,32 @@ void CCodeGen_x86::CommitSymbolRegisterMdAvx(CSymbol* symbol, CX86Assembler::XMM
 	}
 }
 
+bool CCodeGen_x86::TryExpandConstantInRegisterMdAvx(uint32 constant, CX86Assembler::XMMREGISTER dstRegister)
+{
+	switch(constant)
+	{
+	case 0x00000000:
+		m_assembler.VpxorVo(dstRegister, dstRegister, CX86Assembler::MakeXmmRegisterAddress(dstRegister));
+		return true;
+	case 0x3F800000:
+		m_assembler.VmovapsVo(dstRegister, MakeConstant128Address(g_fpCstOne));
+		return true;
+	case 0x7FFFFFFF:
+		m_assembler.VpcmpeqdVo(dstRegister, dstRegister, CX86Assembler::MakeXmmRegisterAddress(dstRegister));
+		m_assembler.VpsrldVo(dstRegister, dstRegister, 1);
+		return true;
+	case 0x80000000:
+		m_assembler.VpcmpeqdVo(dstRegister, dstRegister, CX86Assembler::MakeXmmRegisterAddress(dstRegister));
+		m_assembler.VpslldVo(dstRegister, dstRegister, 31);
+		return true;
+	case 0xFFFFFFFF:
+		m_assembler.VpcmpeqdVo(dstRegister, dstRegister, CX86Assembler::MakeXmmRegisterAddress(dstRegister));
+		return true;
+	default:
+		return false;
+	}
+}
+
 template <typename MDOP>
 void CCodeGen_x86::Emit_Md_Avx_VarVar(const STATEMENT& statement)
 {
@@ -149,28 +175,28 @@ void CCodeGen_x86::Emit_Md_Avx_AddSSW_VarVarVar(const STATEMENT& statement)
 	auto resRegister = CX86Assembler::xMM2;
 	auto cstRegister = CX86Assembler::xMM3;
 
-//	This is based on code from http://locklessinc.com/articles/sat_arithmetic/ modified to work without cmovns
-//	s32b sat_adds32b(s32b x, s32b y)
-//	{
-//		u32b ux = x;
-//		u32b uy = y;
-//		u32b res = ux + uy;
-//	
-//		/* Calculate overflowed result. (Don't change the sign bit of ux) */
-//		ux = (ux >> 31) + INT_MAX;
-//	
-//		s32b sign = (s32b) ((ux ^ uy) | ~(uy ^ res))
-//		sign >>= 31;		/* Arithmetic shift, either 0 or ~0*/
-//		res = (res & sign) | (ux & ~sign);
-//		
-//		return res;
-//	}
+	//	This is based on code from http://locklessinc.com/articles/sat_arithmetic/ modified to work without cmovns
+	//	s32b sat_adds32b(s32b x, s32b y)
+	//	{
+	//		u32b ux = x;
+	//		u32b uy = y;
+	//		u32b res = ux + uy;
+	//
+	//		/* Calculate overflowed result. (Don't change the sign bit of ux) */
+	//		ux = (ux >> 31) + INT_MAX;
+	//
+	//		s32b sign = (s32b) ((ux ^ uy) | ~(uy ^ res))
+	//		sign >>= 31;		/* Arithmetic shift, either 0 or ~0*/
+	//		res = (res & sign) | (ux & ~sign);
+	//
+	//		return res;
+	//	}
 
 	//ux = src1
 	//uy = src2
 	m_assembler.VmovdqaVo(uxRegister, MakeVariable128SymbolAddress(src1));
 	m_assembler.VmovdqaVo(uyRegister, MakeVariable128SymbolAddress(src2));
-	
+
 	//res = ux + uy
 	m_assembler.VpadddVo(resRegister, uxRegister, CX86Assembler::MakeXmmRegisterAddress(uyRegister));
 
@@ -230,26 +256,26 @@ void CCodeGen_x86::Emit_Md_Avx_SubSSW_VarVarVar(const STATEMENT& statement)
 	auto resRegister = CX86Assembler::xMM2;
 	auto cstRegister = CX86Assembler::xMM3;
 
-//	This is based on code from http://locklessinc.com/articles/sat_arithmetic/ modified to work without cmovns
-//	s32b sat_subs32b(s32b x, s32b y)
-//	{
-//		u32b ux = x;
-//		u32b uy = y;
-//		u32b res = ux - uy;
-//		
-//		ux = (ux >> 31) + INT_MAX;
-//		
-//		s32b sign = (s32b) ((ux ^ uy) & (ux ^ res))
-//		sign >>= 31;		/* Arithmetic shift, either 0 or ~0*/
-//		res = (res & ~sign) | (ux & sign);
-//		
-//		return res;
-//	}
+	//	This is based on code from http://locklessinc.com/articles/sat_arithmetic/ modified to work without cmovns
+	//	s32b sat_subs32b(s32b x, s32b y)
+	//	{
+	//		u32b ux = x;
+	//		u32b uy = y;
+	//		u32b res = ux - uy;
+	//
+	//		ux = (ux >> 31) + INT_MAX;
+	//
+	//		s32b sign = (s32b) ((ux ^ uy) & (ux ^ res))
+	//		sign >>= 31;		/* Arithmetic shift, either 0 or ~0*/
+	//		res = (res & ~sign) | (ux & sign);
+	//
+	//		return res;
+	//	}
 
 	//ux = src1
 	//uy = src2
 	m_assembler.VmovdqaVo(uxRegister, MakeVariable128SymbolAddress(src1));
-	
+
 	//res = ux - uy
 	m_assembler.VpsubdVo(resRegister, uxRegister, MakeVariable128SymbolAddress(src2));
 
@@ -305,18 +331,18 @@ void CCodeGen_x86::Emit_Md_Avx_AddUSW_VarVarVar(const STATEMENT& statement)
 	auto tmpRegister = CX86Assembler::xMM2;
 	auto tmp2Register = CX86Assembler::xMM3;
 
-//	This is based on code from http://locklessinc.com/articles/sat_arithmetic/
-//	u32b sat_addu32b(u32b x, u32b y)
-//	{
-//		u32b res = x + y;
-//		res |= -(res < x);
-//	
-//		return res;
-//	}
+	//	This is based on code from http://locklessinc.com/articles/sat_arithmetic/
+	//	u32b sat_addu32b(u32b x, u32b y)
+	//	{
+	//		u32b res = x + y;
+	//		res |= -(res < x);
+	//
+	//		return res;
+	//	}
 
 	m_assembler.VmovdqaVo(xRegister, MakeVariable128SymbolAddress(src1));
 	m_assembler.VpadddVo(resRegister, xRegister, MakeVariable128SymbolAddress(src2));
-	
+
 	//-(res < x)
 	//PCMPGT will compare two signed integers, but we want unsigned comparison
 	//Thus, we add 0x80000000 to both values to "convert" them to signed
@@ -348,14 +374,14 @@ void CCodeGen_x86::Emit_Md_Avx_SubUSW_VarVarVar(const STATEMENT& statement)
 	auto tmpRegister = CX86Assembler::xMM2;
 	auto tmp2Register = CX86Assembler::xMM3;
 
-//	This is based on code from http://locklessinc.com/articles/sat_arithmetic/
-//	u32b sat_subu32b(u32b x, u32b y)
-//	{
-//		u32b res = x - y;
-//		res &= -(res <= x);
-//	
-//		return res;
-//	}
+	//	This is based on code from http://locklessinc.com/articles/sat_arithmetic/
+	//	u32b sat_subu32b(u32b x, u32b y)
+	//	{
+	//		u32b res = x - y;
+	//		res &= -(res <= x);
+	//
+	//		return res;
+	//	}
 
 	m_assembler.VmovdqaVo(xRegister, MakeVariable128SymbolAddress(src1));
 	m_assembler.VpsubdVo(resRegister, xRegister, MakeVariable128SymbolAddress(src2));
@@ -480,34 +506,79 @@ void CCodeGen_x86::Emit_Md_Avx_Abs_VarVar(const STATEMENT& statement)
 	CommitSymbolRegisterMdAvx(dst, dstRegister);
 }
 
+void CCodeGen_x86::Emit_Md_Avx_Neg_VarVar(const STATEMENT& statement)
+{
+	auto dst = statement.dst->GetSymbol().get();
+	auto src1 = statement.src1->GetSymbol().get();
+
+	auto dstRegister = PrepareSymbolRegisterDefMd(dst, CX86Assembler::xMM0);
+	auto bitRegister = CX86Assembler::xMM1;
+
+	assert(dstRegister != bitRegister);
+
+	m_assembler.VpcmpeqdVo(bitRegister, bitRegister, CX86Assembler::MakeXmmRegisterAddress(bitRegister));
+	m_assembler.VpslldVo(bitRegister, bitRegister, 31);
+	m_assembler.VpxorVo(dstRegister, bitRegister, MakeVariable128SymbolAddress(src1));
+
+	CommitSymbolRegisterMdAvx(dst, dstRegister);
+}
+
+void CCodeGen_x86::Emit_Md_Avx_MakeClip_VarVarVarVar(const STATEMENT& statement)
+{
+	auto dst = statement.dst->GetSymbol().get();
+	auto src1 = statement.src1->GetSymbol().get();
+	auto src2 = statement.src2->GetSymbol().get();
+	auto src3 = statement.src3->GetSymbol().get();
+
+	auto dstRegister = PrepareSymbolRegisterDef(dst, CX86Assembler::rDX);
+	auto valueReg = PrepareSymbolRegisterUseMdAvx(src1, CX86Assembler::xMM0);
+
+	auto gtReg = CX86Assembler::xMM1;
+	auto ltReg = CX86Assembler::xMM2;
+	auto tmpReg = CX86Assembler::xMM3;
+
+	//Comparisons
+	m_assembler.VcmpgtpsVo(gtReg, valueReg, MakeVariable128SymbolAddress(src2));
+	m_assembler.VcmpltpsVo(ltReg, valueReg, MakeVariable128SymbolAddress(src3));
+
+	//Pack
+	m_assembler.VpackssdwVo(tmpReg, gtReg, CX86Assembler::MakeXmmRegisterAddress(ltReg));
+
+	//Extract bits
+	m_assembler.VpshufbVo(gtReg, tmpReg, MakeConstant128Address(g_makeClipShufflePattern));
+	m_assembler.VpmovmskbVo(dstRegister, gtReg);
+
+	CommitSymbolRegister(dst, dstRegister);
+}
+
 void CCodeGen_x86::Emit_Md_Avx_MakeSz_VarVar(const STATEMENT& statement)
 {
 	auto dst = statement.dst->GetSymbol().get();
 	auto src1 = statement.src1->GetSymbol().get();
-	
+
 	auto dstRegister = PrepareSymbolRegisterDef(dst, CX86Assembler::rDX);
 	auto src1Register = PrepareSymbolRegisterUseMdAvx(src1, CX86Assembler::xMM0);
 	auto szRegister = CX86Assembler::xMM1;
 	auto zeroRegister = CX86Assembler::xMM2;
-	
+
 	//Compute sign
 	m_assembler.VpsradVo(szRegister, src1Register, 31);
-	
+
 	//Compute zero
 	m_assembler.VpxorVo(zeroRegister, zeroRegister, CX86Assembler::MakeXmmRegisterAddress(zeroRegister));
 	m_assembler.VcmppsVo(zeroRegister, zeroRegister, CX86Assembler::MakeXmmRegisterAddress(src1Register), CX86Assembler::SSE_CMP_EQ);
-	
+
 	//Pack
 	m_assembler.VpackssdwVo(szRegister, szRegister, CX86Assembler::MakeXmmRegisterAddress(zeroRegister));
-	
+
 	//Extract bits
 	m_assembler.VpshufbVo(szRegister, szRegister, MakeConstant128Address(g_makeSzShufflePattern));
 	m_assembler.VpmovmskbVo(dstRegister, szRegister);
-	
+
 	CommitSymbolRegister(dst, dstRegister);
 }
 
-void CCodeGen_x86::Emit_Md_Avx_Expand_VarVar(const STATEMENT& statement)
+void CCodeGen_x86::Emit_Md_Avx_ExpandW_VarVar(const STATEMENT& statement)
 {
 	auto dst = statement.dst->GetSymbol().get();
 	auto src1 = statement.src1->GetSymbol().get();
@@ -520,18 +591,14 @@ void CCodeGen_x86::Emit_Md_Avx_Expand_VarVar(const STATEMENT& statement)
 	CommitSymbolRegisterMdAvx(dst, dstRegister);
 }
 
-void CCodeGen_x86::Emit_Md_Avx_Expand_VarCst(const STATEMENT& statement)
+void CCodeGen_x86::Emit_Md_Avx_ExpandW_VarCst(const STATEMENT& statement)
 {
 	auto dst = statement.dst->GetSymbol().get();
 	auto src1 = statement.src1->GetSymbol().get();
 
 	auto dstRegister = PrepareSymbolRegisterDefMd(dst, CX86Assembler::xMM0);
 
-	if(src1->m_valueLow == 0)
-	{
-		m_assembler.VpxorVo(dstRegister, dstRegister, CX86Assembler::MakeXmmRegisterAddress(dstRegister));
-	}
-	else
+	if(!TryExpandConstantInRegisterMdAvx(src1->m_valueLow, dstRegister))
 	{
 		auto cstRegister = CX86Assembler::rAX;
 		m_assembler.MovId(cstRegister, src1->m_valueLow);
@@ -542,7 +609,22 @@ void CCodeGen_x86::Emit_Md_Avx_Expand_VarCst(const STATEMENT& statement)
 	CommitSymbolRegisterMdAvx(dst, dstRegister);
 }
 
-void CCodeGen_x86::Emit_Md_Avx2_Expand_VarReg(const STATEMENT& statement)
+void CCodeGen_x86::Emit_Md_Avx_ExpandW_VarVarCst(const STATEMENT& statement)
+{
+	auto dst = statement.dst->GetSymbol().get();
+	auto src1 = statement.src1->GetSymbol().get();
+	auto src2 = statement.src2->GetSymbol().get();
+
+	auto dstRegister = PrepareSymbolRegisterDefMd(dst, CX86Assembler::xMM0);
+	auto srcRegister = PrepareSymbolRegisterUseMdAvx(src1, CX86Assembler::xMM1);
+	uint8 shufImm = g_mdExpandShufPatterns[src2->m_valueLow];
+
+	m_assembler.VshufpsVo(dstRegister, srcRegister, CX86Assembler::MakeXmmRegisterAddress(srcRegister), shufImm);
+
+	CommitSymbolRegisterMdAvx(dst, dstRegister);
+}
+
+void CCodeGen_x86::Emit_Md_Avx2_ExpandW_VarReg(const STATEMENT& statement)
 {
 	auto dst = statement.dst->GetSymbol().get();
 	auto src1 = statement.src1->GetSymbol().get();
@@ -555,7 +637,7 @@ void CCodeGen_x86::Emit_Md_Avx2_Expand_VarReg(const STATEMENT& statement)
 	CommitSymbolRegisterMdAvx(dst, dstRegister);
 }
 
-void CCodeGen_x86::Emit_Md_Avx2_Expand_VarMem(const STATEMENT& statement)
+void CCodeGen_x86::Emit_Md_Avx2_ExpandW_VarMem(const STATEMENT& statement)
 {
 	auto dst = statement.dst->GetSymbol().get();
 	auto src1 = statement.src1->GetSymbol().get();
@@ -567,18 +649,14 @@ void CCodeGen_x86::Emit_Md_Avx2_Expand_VarMem(const STATEMENT& statement)
 	CommitSymbolRegisterMdAvx(dst, dstRegister);
 }
 
-void CCodeGen_x86::Emit_Md_Avx2_Expand_VarCst(const STATEMENT& statement)
+void CCodeGen_x86::Emit_Md_Avx2_ExpandW_VarCst(const STATEMENT& statement)
 {
 	auto dst = statement.dst->GetSymbol().get();
 	auto src1 = statement.src1->GetSymbol().get();
 
 	auto dstRegister = PrepareSymbolRegisterDefMd(dst, CX86Assembler::xMM0);
 
-	if(src1->m_valueLow == 0)
-	{
-		m_assembler.VpxorVo(dstRegister, dstRegister, CX86Assembler::MakeXmmRegisterAddress(dstRegister));
-	}
-	else
+	if(!TryExpandConstantInRegisterMdAvx(src1->m_valueLow, dstRegister))
 	{
 		auto cstRegister = CX86Assembler::rAX;
 		m_assembler.MovId(cstRegister, src1->m_valueLow);
@@ -589,6 +667,39 @@ void CCodeGen_x86::Emit_Md_Avx2_Expand_VarCst(const STATEMENT& statement)
 	CommitSymbolRegisterMdAvx(dst, dstRegister);
 }
 
+void CCodeGen_x86::Emit_Md_Avx2_ExpandW_VarRegCst(const STATEMENT& statement)
+{
+	auto dst = statement.dst->GetSymbol().get();
+	auto src1 = statement.src1->GetSymbol().get();
+	auto src2 = statement.src2->GetSymbol().get();
+
+	auto dstRegister = PrepareSymbolRegisterDefMd(dst, CX86Assembler::xMM0);
+	auto srcRegister = m_mdRegisters[src1->m_valueLow];
+
+	if(src2->m_valueLow == 0)
+	{
+		m_assembler.VpbroadcastdVo(dstRegister, CX86Assembler::MakeXmmRegisterAddress(srcRegister));
+	}
+	else
+	{
+		uint8 shufImm = g_mdExpandShufPatterns[src2->m_valueLow];
+		m_assembler.VshufpsVo(dstRegister, srcRegister, CX86Assembler::MakeXmmRegisterAddress(srcRegister), shufImm);
+	}
+
+	CommitSymbolRegisterMdAvx(dst, dstRegister);
+}
+
+void CCodeGen_x86::Emit_Md_Avx2_ExpandW_VarMemCst(const STATEMENT& statement)
+{
+	auto dst = statement.dst->GetSymbol().get();
+	auto src1 = statement.src1->GetSymbol().get();
+	auto src2 = statement.src2->GetSymbol().get();
+
+	auto dstRegister = PrepareSymbolRegisterDefMd(dst, CX86Assembler::xMM0);
+	m_assembler.VpbroadcastdVo(dstRegister, MakeMemory128SymbolElementAddress(src1, src2->m_valueLow));
+	CommitSymbolRegisterMdAvx(dst, dstRegister);
+}
+
 void CCodeGen_x86::Emit_Avx_MergeTo256_MemVarVar(const STATEMENT& statement)
 {
 	auto dst = statement.dst->GetSymbol().get();
@@ -698,6 +809,37 @@ void CCodeGen_x86::Emit_Md_Avx_StoreAtRef_VarAnyVar(const STATEMENT& statement)
 	m_assembler.VmovapsVo(MakeRefBaseScaleSymbolAddress(src1, CX86Assembler::rAX, src2, CX86Assembler::rCX, scale), valueReg);
 }
 
+void CCodeGen_x86::Emit_Md_Avx_LoadFromRefMasked_VarVarAnyVar(const STATEMENT& statement)
+{
+	auto dst = statement.dst->GetSymbol().get();
+	auto src1 = statement.src1->GetSymbol().get();
+	auto src2 = statement.src2->GetSymbol().get();
+	auto src3 = statement.src3->GetSymbol().get();
+	uint8 mask = static_cast<uint8>(statement.jmpCondition);
+
+	auto dstReg = PrepareSymbolRegisterDefMd(dst, CX86Assembler::xMM0);
+	auto src3Reg = PrepareSymbolRegisterUseMdAvx(src3, CX86Assembler::xMM1);
+
+	m_assembler.VblendpsVo(dstReg, src3Reg, MakeRefBaseScaleSymbolAddress(src1, CX86Assembler::rAX, src2, CX86Assembler::rCX, 1), mask);
+	CommitSymbolRegisterMdAvx(dst, dstReg);
+}
+
+void CCodeGen_x86::Emit_Md_Avx_StoreAtRefMasked_VarAnyVar(const STATEMENT& statement)
+{
+	auto src1 = statement.src1->GetSymbol().get();
+	auto src2 = statement.src2->GetSymbol().get();
+	auto src3 = statement.src3->GetSymbol().get();
+	uint8 mask = static_cast<uint8>(statement.jmpCondition);
+
+	auto valueReg = PrepareSymbolRegisterUseMdAvx(src3, CX86Assembler::xMM0);
+	auto tmpReg = CX86Assembler::xMM1;
+	auto dstAddress = MakeRefBaseScaleSymbolAddress(src1, CX86Assembler::rAX, src2, CX86Assembler::rCX, 1);
+
+	m_assembler.VblendpsVo(tmpReg, valueReg, dstAddress, ~mask & 0x0F);
+	m_assembler.VmovapsVo(dstAddress, tmpReg);
+}
+
+// clang-format off
 CCodeGen_x86::CONSTMATCHER CCodeGen_x86::g_mdAvxConstMatchers[] = 
 {
 	{ OP_MD_ADD_B, MATCH_VARIABLE128, MATCH_VARIABLE128, MATCH_VARIABLE128, MATCH_NIL, &CCodeGen_x86::Emit_Md_Avx_VarVarVar<MDOP_ADDB> },
@@ -767,6 +909,7 @@ CCodeGen_x86::CONSTMATCHER CCodeGen_x86::g_mdAvxConstMatchers[] =
 	{ OP_MD_DIV_S, MATCH_VARIABLE128, MATCH_VARIABLE128, MATCH_VARIABLE128, MATCH_NIL, &CCodeGen_x86::Emit_Md_Avx_VarVarVar<MDOP_DIVS> },
 
 	{ OP_MD_ABS_S, MATCH_VARIABLE128, MATCH_VARIABLE128, MATCH_NIL, MATCH_NIL, &CCodeGen_x86::Emit_Md_Avx_Abs_VarVar },
+	{ OP_MD_NEG_S, MATCH_VARIABLE128, MATCH_VARIABLE128, MATCH_NIL, MATCH_NIL, &CCodeGen_x86::Emit_Md_Avx_Neg_VarVar },
 
 	{ OP_MD_CMPLT_S, MATCH_VARIABLE128, MATCH_VARIABLE128, MATCH_VARIABLE128, MATCH_NIL, &CCodeGen_x86::Emit_Md_Avx_VarVarVar<MDOP_CMPLTS> },
 	{ OP_MD_CMPGT_S, MATCH_VARIABLE128, MATCH_VARIABLE128, MATCH_VARIABLE128, MATCH_NIL, &CCodeGen_x86::Emit_Md_Avx_VarVarVar<MDOP_CMPGTS> },
@@ -774,13 +917,14 @@ CCodeGen_x86::CONSTMATCHER CCodeGen_x86::g_mdAvxConstMatchers[] =
 	{ OP_MD_MIN_S, MATCH_VARIABLE128, MATCH_VARIABLE128, MATCH_VARIABLE128, MATCH_NIL, &CCodeGen_x86::Emit_Md_Avx_VarVarVar<MDOP_MINS> },
 	{ OP_MD_MAX_S, MATCH_VARIABLE128, MATCH_VARIABLE128, MATCH_VARIABLE128, MATCH_NIL, &CCodeGen_x86::Emit_Md_Avx_VarVarVar<MDOP_MAXS> },
 
-	{ OP_MD_TOWORD_TRUNCATE, MATCH_VARIABLE128, MATCH_VARIABLE128, MATCH_NIL, MATCH_NIL, &CCodeGen_x86::Emit_Md_Avx_VarVar<MDOP_TOWORD_TRUNCATE> },
-	{ OP_MD_TOSINGLE,        MATCH_VARIABLE128, MATCH_VARIABLE128, MATCH_NIL, MATCH_NIL, &CCodeGen_x86::Emit_Md_Avx_VarVar<MDOP_TOSINGLE>        },
+	{ OP_MD_TOINT32_TRUNC_S, MATCH_VARIABLE128, MATCH_VARIABLE128, MATCH_NIL, MATCH_NIL, &CCodeGen_x86::Emit_Md_Avx_VarVar<MDOP_TOINT32_TRUNC_S> },
+	{ OP_MD_TOSINGLE_I32,    MATCH_VARIABLE128, MATCH_VARIABLE128, MATCH_NIL, MATCH_NIL, &CCodeGen_x86::Emit_Md_Avx_VarVar<MDOP_TOSINGLE_I32>    },
 
 	{ OP_MD_PACK_HB, MATCH_VARIABLE128, MATCH_VARIABLE128, MATCH_VARIABLE128, MATCH_NIL, &CCodeGen_x86::Emit_Md_Avx_PackHB_VarVarVar, },
 	{ OP_MD_PACK_WH, MATCH_VARIABLE128, MATCH_VARIABLE128, MATCH_VARIABLE128, MATCH_NIL, &CCodeGen_x86::Emit_Md_Avx_PackWH_VarVarVar, },
 
-	{ OP_MD_MAKESZ, MATCH_VARIABLE, MATCH_VARIABLE128, MATCH_NIL, MATCH_NIL, &CCodeGen_x86::Emit_Md_Avx_MakeSz_VarVar },
+	{ OP_MD_MAKECLIP, MATCH_VARIABLE, MATCH_VARIABLE128, MATCH_VARIABLE128, MATCH_VARIABLE128, &CCodeGen_x86::Emit_Md_Avx_MakeClip_VarVarVarVar },
+	{ OP_MD_MAKESZ,   MATCH_VARIABLE, MATCH_VARIABLE128, MATCH_NIL,         MATCH_NIL,         &CCodeGen_x86::Emit_Md_Avx_MakeSz_VarVar },
 
 	{ OP_MOV, MATCH_REGISTER128, MATCH_VARIABLE128, MATCH_NIL, MATCH_NIL, &CCodeGen_x86::Emit_Md_Avx_Mov_RegVar, },
 	{ OP_MOV, MATCH_MEMORY128,   MATCH_REGISTER128, MATCH_NIL, MATCH_NIL, &CCodeGen_x86::Emit_Md_Avx_Mov_MemReg, },
@@ -798,22 +942,31 @@ CCodeGen_x86::CONSTMATCHER CCodeGen_x86::g_mdAvxConstMatchers[] =
 	{ OP_STOREATREF, MATCH_NIL, MATCH_VAR_REF, MATCH_VARIABLE128, MATCH_NIL,         &CCodeGen_x86::Emit_Md_Avx_StoreAtRef_VarVar },
 	{ OP_STOREATREF, MATCH_NIL, MATCH_VAR_REF, MATCH_ANY32,       MATCH_VARIABLE128, &CCodeGen_x86::Emit_Md_Avx_StoreAtRef_VarAnyVar },
 
-	{ OP_MOV, MATCH_NIL,         MATCH_NIL,         MATCH_NIL, MATCH_NIL, nullptr },
-};
-
-CCodeGen_x86::CONSTMATCHER CCodeGen_x86::g_mdAvxExpandConstMatchers[] =
-{
-	{ OP_MD_EXPAND, MATCH_VARIABLE128, MATCH_VARIABLE, MATCH_NIL, MATCH_NIL, &CCodeGen_x86::Emit_Md_Avx_Expand_VarVar },
-	{ OP_MD_EXPAND, MATCH_VARIABLE128, MATCH_CONSTANT, MATCH_NIL, MATCH_NIL, &CCodeGen_x86::Emit_Md_Avx_Expand_VarCst },
+	{ OP_MD_LOADFROMREF_MASKED, MATCH_VARIABLE128, MATCH_VAR_REF, MATCH_ANY32, MATCH_VARIABLE128, &CCodeGen_x86::Emit_Md_Avx_LoadFromRefMasked_VarVarAnyVar },
+	{ OP_MD_STOREATREF_MASKED,  MATCH_NIL,         MATCH_VAR_REF, MATCH_ANY32, MATCH_VARIABLE128, &CCodeGen_x86::Emit_Md_Avx_StoreAtRefMasked_VarAnyVar     },
 
 	{ OP_MOV, MATCH_NIL,         MATCH_NIL,         MATCH_NIL, MATCH_NIL, nullptr },
 };
 
-CCodeGen_x86::CONSTMATCHER CCodeGen_x86::g_mdAvx2ExpandConstMatchers[] =
+CCodeGen_x86::CONSTMATCHER CCodeGen_x86::g_mdNoAvx2ConstMatchers[] =
 {
-	{ OP_MD_EXPAND, MATCH_VARIABLE128, MATCH_REGISTER, MATCH_NIL, MATCH_NIL, &CCodeGen_x86::Emit_Md_Avx2_Expand_VarReg },
-	{ OP_MD_EXPAND, MATCH_VARIABLE128, MATCH_MEMORY,   MATCH_NIL, MATCH_NIL, &CCodeGen_x86::Emit_Md_Avx2_Expand_VarMem },
-	{ OP_MD_EXPAND, MATCH_VARIABLE128, MATCH_CONSTANT, MATCH_NIL, MATCH_NIL, &CCodeGen_x86::Emit_Md_Avx2_Expand_VarCst },
+	{ OP_MD_EXPAND_W, MATCH_VARIABLE128, MATCH_VARIABLE, MATCH_NIL, MATCH_NIL, &CCodeGen_x86::Emit_Md_Avx_ExpandW_VarVar },
+	{ OP_MD_EXPAND_W, MATCH_VARIABLE128, MATCH_CONSTANT, MATCH_NIL, MATCH_NIL, &CCodeGen_x86::Emit_Md_Avx_ExpandW_VarCst },
+
+	{ OP_MD_EXPAND_W, MATCH_VARIABLE128, MATCH_VARIABLE128, MATCH_CONSTANT, MATCH_NIL, &CCodeGen_x86::Emit_Md_Avx_ExpandW_VarVarCst },
+
+	{ OP_MOV, MATCH_NIL,         MATCH_NIL,         MATCH_NIL, MATCH_NIL, nullptr },
+};
+
+CCodeGen_x86::CONSTMATCHER CCodeGen_x86::g_mdAvx2ConstMatchers[] =
+{
+	{ OP_MD_EXPAND_W, MATCH_VARIABLE128, MATCH_REGISTER, MATCH_NIL, MATCH_NIL, &CCodeGen_x86::Emit_Md_Avx2_ExpandW_VarReg },
+	{ OP_MD_EXPAND_W, MATCH_VARIABLE128, MATCH_MEMORY,   MATCH_NIL, MATCH_NIL, &CCodeGen_x86::Emit_Md_Avx2_ExpandW_VarMem },
+	{ OP_MD_EXPAND_W, MATCH_VARIABLE128, MATCH_CONSTANT, MATCH_NIL, MATCH_NIL, &CCodeGen_x86::Emit_Md_Avx2_ExpandW_VarCst },
+
+	{ OP_MD_EXPAND_W, MATCH_VARIABLE128, MATCH_REGISTER128, MATCH_CONSTANT, MATCH_NIL, &CCodeGen_x86::Emit_Md_Avx2_ExpandW_VarRegCst },
+	{ OP_MD_EXPAND_W, MATCH_VARIABLE128, MATCH_MEMORY128,   MATCH_CONSTANT, MATCH_NIL, &CCodeGen_x86::Emit_Md_Avx2_ExpandW_VarMemCst },
 
 	{ OP_MOV, MATCH_NIL, MATCH_NIL, MATCH_NIL, MATCH_NIL, nullptr },
 };
+// clang-format on
