@@ -14,12 +14,6 @@
 
 using namespace Jitter;
 
-static uint64 MergeConstant64(uint32 lo, uint32 hi)
-{
-	uint64 result = static_cast<uint64>(lo) | (static_cast<uint64>(hi) >> 32);
-	return result;
-}
-
 unsigned int CJitter::CRelativeVersionManager::GetRelativeVersion(uint32 relativeId)
 {
 	RelativeVersionMap::const_iterator versionIterator(m_relativeVersions.find(relativeId));
@@ -40,17 +34,42 @@ CJitter::VERSIONED_STATEMENT_LIST CJitter::GenerateVersionedStatementList(const
 
 	struct ReplaceUse
 	{
-		void operator() (SymbolRefPtr& symbolRef, CRelativeVersionManager& relativeVersions) const
+		void operator()(SymbolRefPtr& symbolRef, CRelativeVersionManager& relativeVersions) const
 		{
 			if(CSymbol* symbol = dynamic_symbolref_cast(SYM_RELATIVE, symbolRef))
 			{
 				unsigned int currentVersion = relativeVersions.GetRelativeVersion(symbol->m_valueLow);
-				symbolRef = std::make_shared<CVersionedSymbolRef>(symbolRef->GetSymbol(), currentVersion);
+				symbolRef = std::make_shared<CSymbolRef>(symbolRef->GetSymbol(), currentVersion);
 			}
-			if(CSymbol* symbol = dynamic_symbolref_cast(SYM_RELATIVE64, symbolRef))
+			else if(CSymbol* symbol = dynamic_symbolref_cast(SYM_REL_REFERENCE, symbolRef))
 			{
 				unsigned int currentVersion = relativeVersions.GetRelativeVersion(symbol->m_valueLow);
-				symbolRef = std::make_shared<CVersionedSymbolRef>(symbolRef->GetSymbol(), currentVersion);
+				symbolRef = std::make_shared<CSymbolRef>(symbolRef->GetSymbol(), currentVersion);
+			}
+			else if(CSymbol* symbol = dynamic_symbolref_cast(SYM_RELATIVE64, symbolRef))
+			{
+				//Since this symbol can be aliased, use the sum of the versions of all
+				//of its parts.
+				unsigned int currentVersion =
+				    relativeVersions.GetRelativeVersion(symbol->m_valueLow + 0x0) +
+				    relativeVersions.GetRelativeVersion(symbol->m_valueLow + 0x4);
+				symbolRef = std::make_shared<CSymbolRef>(symbolRef->GetSymbol(), currentVersion);
+			}
+			else if(CSymbol* symbol = dynamic_symbolref_cast(SYM_FP_RELATIVE32, symbolRef))
+			{
+				unsigned int currentVersion = relativeVersions.GetRelativeVersion(symbol->m_valueLow);
+				symbolRef = std::make_shared<CSymbolRef>(symbolRef->GetSymbol(), currentVersion);
+			}
+			else if(CSymbol* symbol = dynamic_symbolref_cast(SYM_RELATIVE128, symbolRef))
+			{
+				//Since this symbol can be aliased, use the sum of the versions of all
+				//of its parts.
+				unsigned int currentVersion =
+				    relativeVersions.GetRelativeVersion(symbol->m_valueLow + 0x0) +
+				    relativeVersions.GetRelativeVersion(symbol->m_valueLow + 0x4) +
+				    relativeVersions.GetRelativeVersion(symbol->m_valueLow + 0x8) +
+				    relativeVersions.GetRelativeVersion(symbol->m_valueLow + 0xC);
+				symbolRef = std::make_shared<CSymbolRef>(symbolRef->GetSymbol(), currentVersion);
 			}
 		}
 	};
@@ -61,26 +80,26 @@ CJitter::VERSIONED_STATEMENT_LIST CJitter::GenerateVersionedStatementList(const
 		ReplaceUse()(newStatement.src2, result.relativeVersions);
 		ReplaceUse()(newStatement.src3, result.relativeVersions);
 
-		if(CSymbol* dst = dynamic_symbolref_cast(SYM_RELATIVE, newStatement.dst))
+		if(auto dst = dynamic_symbolref_cast(SYM_RELATIVE, newStatement.dst))
 		{
 			unsigned int nextVersion = result.relativeVersions.IncrementRelativeVersion(dst->m_valueLow);
-			newStatement.dst = std::make_shared<CVersionedSymbolRef>(newStatement.dst->GetSymbol(), nextVersion);
+			newStatement.dst = std::make_shared<CSymbolRef>(newStatement.dst->GetSymbol(), nextVersion);
 		}
 		//Increment relative versions to prevent some optimization problems
-		else if(CSymbol* dst = dynamic_symbolref_cast(SYM_FP_REL_SINGLE, newStatement.dst))
+		else if(auto dst = dynamic_symbolref_cast(SYM_REL_REFERENCE, newStatement.dst))
 		{
 			result.relativeVersions.IncrementRelativeVersion(dst->m_valueLow);
 		}
-		else if(CSymbol* dst = dynamic_symbolref_cast(SYM_FP_REL_INT32, newStatement.dst))
+		else if(auto dst = dynamic_symbolref_cast(SYM_FP_RELATIVE32, newStatement.dst))
 		{
 			result.relativeVersions.IncrementRelativeVersion(dst->m_valueLow);
 		}
-		else if(CSymbol* dst = dynamic_symbolref_cast(SYM_RELATIVE64, newStatement.dst))
+		else if(auto dst = dynamic_symbolref_cast(SYM_RELATIVE64, newStatement.dst))
 		{
 			result.relativeVersions.IncrementRelativeVersion(dst->m_valueLow + 0);
 			result.relativeVersions.IncrementRelativeVersion(dst->m_valueLow + 4);
 		}
-		else if(CSymbol* dst = dynamic_symbolref_cast(SYM_RELATIVE128, newStatement.dst))
+		else if(auto dst = dynamic_symbolref_cast(SYM_RELATIVE128, newStatement.dst))
 		{
 			uint8 mask = 0xF;
 			if(newStatement.op == OP_MD_MOV_MASKED)
@@ -88,7 +107,7 @@ CJitter::VERSIONED_STATEMENT_LIST CJitter::GenerateVersionedStatementList(const
 				mask = static_cast<uint8>(newStatement.jmpCondition);
 			}
 
-			if(mask & 0x01) result.relativeVersions.IncrementRelativeVersion(dst->m_valueLow + 0);
+			result.relativeVersions.IncrementRelativeVersion(dst->m_valueLow + 0);
 			if(mask & 0x02) result.relativeVersions.IncrementRelativeVersion(dst->m_valueLow + 4);
 			if(mask & 0x04) result.relativeVersions.IncrementRelativeVersion(dst->m_valueLow + 8);
 			if(mask & 0x08) result.relativeVersions.IncrementRelativeVersion(dst->m_valueLow + 12);
@@ -106,15 +125,13 @@ StatementList CJitter::CollapseVersionedStatementList(const VERSIONED_STATEMENT_
 	for(auto newStatement : statements.statements)
 	{
 		newStatement.VisitOperands(
-			[](SymbolRefPtr& symbolRef, bool)
-			{
-				if(auto versionedSymbolRef = std::dynamic_pointer_cast<CVersionedSymbolRef>(symbolRef))
-				{
-					symbolRef = std::make_shared<CSymbolRef>(symbolRef->GetSymbol());
-				}
-			}
-		);
-		
+		    [](SymbolRefPtr& symbolRef, bool) {
+			    if(symbolRef->IsVersioned())
+			    {
+				    symbolRef = std::make_shared<CSymbolRef>(symbolRef->GetSymbol());
+			    }
+		    });
+
 		result.push_back(newStatement);
 	}
 	return result;
@@ -132,6 +149,13 @@ void CJitter::Compile()
 
 				//DumpStatementList(m_currentBlock->statements);
 
+				//These don't need to be run more than once
+				ClampingElimination(basicBlock.statements);
+				if(m_codeGenSupportsCmpSelect)
+				{
+					MergeCmpSelectOps(basicBlock.statements);
+				}
+
 				auto versionedStatements = GenerateVersionedStatementList(basicBlock.statements);
 
 				while(1)
@@ -142,6 +166,7 @@ void CJitter::Compile()
 					dirty |= ReorderAdd(versionedStatements.statements);
 					dirty |= CopyPropagation(versionedStatements.statements);
 					dirty |= DeadcodeElimination(versionedStatements);
+					dirty |= CommonExpressionElimination(versionedStatements);
 
 					if(!dirty) break;
 				}
@@ -201,10 +226,10 @@ SymbolPtr CJitter::MakeSymbol(SYM_TYPE type, uint32 value)
 
 SymbolPtr CJitter::MakeConstantPtr(uintptr_t value)
 {
-#if (UINTPTR_MAX == UINT32_MAX)
+#if(UINTPTR_MAX == UINT32_MAX)
 	uint32 valueLo = static_cast<uint32>(value);
 	return MakeSymbol(m_currentBlock, SYM_CONSTANTPTR, valueLo, 0);
-#elif (UINTPTR_MAX == UINT64_MAX)
+#elif(UINTPTR_MAX == UINT64_MAX)
 	uint32 valueLo = static_cast<uint32>(value);
 	uint32 valueHi = static_cast<uint32>(value >> 32);
 	return MakeSymbol(m_currentBlock, SYM_CONSTANTPTR, valueLo, valueHi);
@@ -238,11 +263,12 @@ int CJitter::GetSymbolSize(const SymbolRefPtr& symbolRef)
 
 bool CJitter::FoldConstantOperation(STATEMENT& statement)
 {
-	CSymbol* src1cst = dynamic_symbolref_cast(SYM_CONSTANT, statement.src1);
-	CSymbol* src2cst = dynamic_symbolref_cast(SYM_CONSTANT, statement.src2);
+	auto src1cst = dynamic_symbolref_cast(SYM_CONSTANT, statement.src1);
+	auto src2cst = dynamic_symbolref_cast(SYM_CONSTANT, statement.src2);
+	auto src3cst = dynamic_symbolref_cast(SYM_CONSTANT, statement.src3);
 
 	//Nothing we can do
-	if(src1cst == NULL && src2cst == NULL) return false;
+	if(!src1cst && !src2cst && !src3cst) return false;
 
 	bool changed = false;
 
@@ -301,9 +327,8 @@ bool CJitter::FoldConstantOperation(STATEMENT& statement)
 			changed = true;
 		}
 		else if(
-			(src1cst && src1cst->m_valueLow == 0) ||
-			(src2cst && src2cst->m_valueLow == 0)
-		)
+		    (src1cst && src1cst->m_valueLow == 0) ||
+		    (src2cst && src2cst->m_valueLow == 0))
 		{
 			//Anding with zero
 			statement.op = OP_MOV;
@@ -457,7 +482,7 @@ bool CJitter::FoldConstantOperation(STATEMENT& statement)
 	{
 		if(src1cst && src2cst)
 		{
-			uint64 result = MergeConstant64(src1cst->m_valueLow, src2cst->m_valueLow);
+			uint64 result = static_cast<uint64>(src1cst->m_valueLow) | (static_cast<uint64>(src2cst->m_valueLow) << 32);
 			statement.op = OP_MOV;
 			statement.src1 = MakeSymbolRef(MakeConstant64(result));
 			statement.src2.reset();
@@ -512,9 +537,9 @@ bool CJitter::FoldConstantOperation(STATEMENT& statement)
 		//	src2cst->m_valueLow = GetPowerOf2(src2cst->m_valueLow);
 		//}
 	}
-	else if (statement.op == OP_DIVS)
+	else if(statement.op == OP_DIVS)
 	{
-		if (src1cst && src2cst)
+		if(src1cst && src2cst)
 		{
 			uint64 result = 0;
 			if(src2cst->m_valueLow != 0)
@@ -540,9 +565,24 @@ bool CJitter::FoldConstantOperation(STATEMENT& statement)
 			bool result = false;
 			switch(statement.jmpCondition)
 			{
+			case CONDITION_EQ:
+				result = static_cast<int32>(src1cst->m_valueLow) == static_cast<int32>(src2cst->m_valueLow);
+				break;
+			case CONDITION_NE:
+				result = static_cast<int32>(src1cst->m_valueLow) != static_cast<int32>(src2cst->m_valueLow);
+				break;
 			case CONDITION_BL:
 				result = src1cst->m_valueLow < src2cst->m_valueLow;
 				break;
+			case CONDITION_BE:
+				result = src1cst->m_valueLow <= src2cst->m_valueLow;
+				break;
+			case CONDITION_AB:
+				result = src1cst->m_valueLow > src2cst->m_valueLow;
+				break;
+			case CONDITION_AE:
+				result = src1cst->m_valueLow >= src2cst->m_valueLow;
+				break;
 			case CONDITION_LT:
 				result = static_cast<int32>(src1cst->m_valueLow) < static_cast<int32>(src2cst->m_valueLow);
 				break;
@@ -552,8 +592,8 @@ bool CJitter::FoldConstantOperation(STATEMENT& statement)
 			case CONDITION_GT:
 				result = static_cast<int32>(src1cst->m_valueLow) > static_cast<int32>(src2cst->m_valueLow);
 				break;
-			case CONDITION_EQ:
-				result = static_cast<int32>(src1cst->m_valueLow) == static_cast<int32>(src2cst->m_valueLow);
+			case CONDITION_GE:
+				result = static_cast<int32>(src1cst->m_valueLow) >= static_cast<int32>(src2cst->m_valueLow);
 				break;
 			default:
 				assert(0);
@@ -565,6 +605,19 @@ bool CJitter::FoldConstantOperation(STATEMENT& statement)
 			statement.src2.reset();
 		}
 	}
+	else if(statement.op == OP_SELECT)
+	{
+		//If true and false values are the same, we don't care about the test
+		if(src2cst && src3cst && (src2cst->m_valueLow == src3cst->m_valueLow))
+		{
+			uint32 value = src2cst->m_valueLow;
+			statement.op = OP_MOV;
+			statement.src1 = MakeSymbolRef(MakeSymbol(SYM_CONSTANT, value));
+			statement.src2.reset();
+			statement.src3.reset();
+			changed = true;
+		}
+	}
 	else if(statement.op == OP_CONDJMP)
 	{
 		if(src1cst && src2cst)
@@ -643,8 +696,8 @@ bool CJitter::FoldConstant64Operation(STATEMENT& statement)
 	{
 		if(src1cst && src2cst)
 		{
-			uint64 cst1 = MergeConstant64(src1cst->m_valueLow, src1cst->m_valueHigh);
-			uint64 cst2 = MergeConstant64(src2cst->m_valueLow, src2cst->m_valueHigh);
+			uint64 cst1 = src1cst->GetConstant64();
+			uint64 cst2 = src2cst->GetConstant64();
 			uint64 result = cst1 + cst2;
 			statement.op = OP_MOV;
 			statement.src1 = MakeSymbolRef(MakeConstant64(result));
@@ -662,8 +715,8 @@ bool CJitter::FoldConstant64Operation(STATEMENT& statement)
 	{
 		if(src1cst && src2cst)
 		{
-			uint64 cst1 = MergeConstant64(src1cst->m_valueLow, src1cst->m_valueHigh);
-			uint64 cst2 = MergeConstant64(src2cst->m_valueLow, src2cst->m_valueHigh);
+			uint64 cst1 = src1cst->GetConstant64();
+			uint64 cst2 = src2cst->GetConstant64();
 			uint64 result = cst1 - cst2;
 			statement.op = OP_MOV;
 			statement.src1 = MakeSymbolRef(MakeConstant64(result));
@@ -675,8 +728,8 @@ bool CJitter::FoldConstant64Operation(STATEMENT& statement)
 	{
 		if(src1cst && src2cst)
 		{
-			uint64 cst1 = MergeConstant64(src1cst->m_valueLow, src1cst->m_valueHigh);
-			uint64 cst2 = MergeConstant64(src2cst->m_valueLow, src2cst->m_valueHigh);
+			uint64 cst1 = src1cst->GetConstant64();
+			uint64 cst2 = src2cst->GetConstant64();
 			uint64 result = cst1 & cst2;
 			statement.op = OP_MOV;
 			statement.src1 = MakeSymbolRef(MakeConstant64(result));
@@ -684,9 +737,8 @@ bool CJitter::FoldConstant64Operation(STATEMENT& statement)
 			changed = true;
 		}
 		else if(
-			(src1cst && (src1cst->m_valueLow == 0) && (src1cst->m_valueHigh == 0)) ||
-			(src2cst && (src2cst->m_valueLow == 0) && (src2cst->m_valueHigh == 0))
-			)
+		    (src1cst && (src1cst->m_valueLow == 0) && (src1cst->m_valueHigh == 0)) ||
+		    (src2cst && (src2cst->m_valueLow == 0) && (src2cst->m_valueHigh == 0)))
 		{
 			//ANDing anything with 0 gives 0
 			statement.op = OP_MOV;
@@ -750,9 +802,9 @@ bool CJitter::FoldConstant6432Operation(STATEMENT& statement)
 	bool changed = false;
 
 	if(
-		statement.op == OP_SLL64 ||
-		statement.op == OP_SRL64 ||
-		statement.op == OP_SRA64)
+	    statement.op == OP_SLL64 ||
+	    statement.op == OP_SRL64 ||
+	    statement.op == OP_SRA64)
 	{
 		if(src2cst && ((src2cst->m_valueLow & 0x3F) == 0))
 		{
@@ -781,9 +833,9 @@ bool CJitter::FoldConstant12832Operation(STATEMENT& statement)
 	bool changed = false;
 
 	if(
-		statement.op == OP_MD_SLLH ||
-		statement.op == OP_MD_SRLH ||
-		statement.op == OP_MD_SRAH)
+	    statement.op == OP_MD_SLLH ||
+	    statement.op == OP_MD_SRLH ||
+	    statement.op == OP_MD_SRAH)
 	{
 		if(src2cst && ((src2cst->m_valueLow & 0xF) == 0))
 		{
@@ -793,9 +845,9 @@ bool CJitter::FoldConstant12832Operation(STATEMENT& statement)
 		}
 	}
 	else if(
-		statement.op == OP_MD_SLLW ||
-		statement.op == OP_MD_SRLW ||
-		statement.op == OP_MD_SRAW)
+	    statement.op == OP_MD_SLLW ||
+	    statement.op == OP_MD_SRLW ||
+	    statement.op == OP_MD_SRAW)
 	{
 		if(src2cst && ((src2cst->m_valueLow & 0x1F) == 0))
 		{
@@ -832,14 +884,14 @@ void CJitter::FixFlowControl(StatementList& statements)
 			assert(labelIterator != m_labels.end());
 			if(labelIterator == m_labels.end()) continue;
 
-			statement.op		= OP_JMP;
-			statement.jmpBlock	= labelIterator->second;
+			statement.op = OP_JMP;
+			statement.jmpBlock = labelIterator->second;
 		}
 	}
 
 	//Remove any excess flow control instructions
 	for(StatementList::iterator statementIterator(statements.begin());
-		statementIterator != statements.end(); ++statementIterator)
+	    statementIterator != statements.end(); ++statementIterator)
 	{
 		const STATEMENT& statement(*statementIterator);
 
@@ -859,12 +911,10 @@ void CJitter::MergeBasicBlocks(BASIC_BLOCK& dstBlock, const BASIC_BLOCK& srcBloc
 	for(auto statement : srcBlock.statements)
 	{
 		statement.VisitOperands(
-			[&dstSymbolTable](SymbolRefPtr& symbolRef, bool)
-			{
-				auto symbol = symbolRef->GetSymbol();
-				symbolRef = std::make_shared<CSymbolRef>(dstSymbolTable.MakeSymbol(symbol));
-			}
-		);
+		    [&dstSymbolTable](SymbolRefPtr& symbolRef, bool) {
+			    auto symbol = symbolRef->GetSymbol();
+			    symbolRef = std::make_shared<CSymbolRef>(dstSymbolTable.MakeSymbol(symbol));
+		    });
 		dstBlock.statements.push_back(statement);
 	}
 
@@ -878,8 +928,8 @@ CJitter::BASIC_BLOCK CJitter::ConcatBlocks(const BasicBlockList& blocks)
 	{
 		//First, add a mark label statement
 		STATEMENT labelStatement;
-		labelStatement.op		= OP_LABEL;
-		labelStatement.jmpBlock	= basicBlock.id;
+		labelStatement.op = OP_LABEL;
+		labelStatement.jmpBlock = basicBlock.id;
 		result.statements.push_back(labelStatement);
 
 		MergeBasicBlocks(result, basicBlock);
@@ -898,7 +948,7 @@ bool CJitter::PruneBlocks()
 
 		auto toDeleteIterator = m_basicBlocks.cend();
 		for(auto outerBlockIterator(m_basicBlocks.cbegin());
-			outerBlockIterator != m_basicBlocks.cend(); outerBlockIterator++)
+		    outerBlockIterator != m_basicBlocks.cend(); outerBlockIterator++)
 		{
 			//First block is always referenced
 			if(outerBlockIterator == m_basicBlocks.begin()) continue;
@@ -908,7 +958,7 @@ bool CJitter::PruneBlocks()
 
 			//Check if there's a reference to this block in here
 			for(auto innerBlockIterator(m_basicBlocks.cbegin());
-				innerBlockIterator != m_basicBlocks.cend(); innerBlockIterator++)
+			    innerBlockIterator != m_basicBlocks.cend(); innerBlockIterator++)
 			{
 				const auto& block(*innerBlockIterator);
 				bool referencesNext = false;
@@ -978,7 +1028,7 @@ void CJitter::HarmonizeBlocks()
 {
 	//Remove any jumps that jump to the next block
 	for(BasicBlockList::iterator blockIterator(m_basicBlocks.begin());
-		blockIterator != m_basicBlocks.end(); ++blockIterator)
+	    blockIterator != m_basicBlocks.end(); ++blockIterator)
 	{
 		BasicBlockList::iterator nextBlockIterator(blockIterator);
 		++nextBlockIterator;
@@ -999,7 +1049,7 @@ void CJitter::HarmonizeBlocks()
 
 	//Flag any block that have a reference from a jump
 	for(BasicBlockList::iterator outerBlockIterator(m_basicBlocks.begin());
-		outerBlockIterator != m_basicBlocks.end(); ++outerBlockIterator)
+	    outerBlockIterator != m_basicBlocks.end(); ++outerBlockIterator)
 	{
 		auto& outerBlock(*outerBlockIterator);
 		outerBlock.hasJumpRef = false;
@@ -1034,7 +1084,7 @@ bool CJitter::MergeBlocks()
 	{
 		changed = false;
 		for(BasicBlockList::iterator blockIterator(m_basicBlocks.begin());
-			m_basicBlocks.end() != blockIterator; ++blockIterator)
+		    m_basicBlocks.end() != blockIterator; ++blockIterator)
 		{
 			BasicBlockList::iterator nextBlockIterator(blockIterator);
 			++nextBlockIterator;
@@ -1073,7 +1123,7 @@ bool CJitter::ConstantPropagation(StatementList& statements)
 	bool changed = false;
 
 	for(StatementList::iterator outerStatementIterator(statements.begin());
-		statements.end() != outerStatementIterator; ++outerStatementIterator)
+	    statements.end() != outerStatementIterator; ++outerStatementIterator)
 	{
 		STATEMENT& outerStatement(*outerStatementIterator);
 
@@ -1088,22 +1138,20 @@ bool CJitter::ConstantPropagation(StatementList& statements)
 
 		//Find anything that uses this operand and replace it with the constant
 		for(StatementList::iterator innerStatementIterator(outerStatementIterator);
-			statements.end() != innerStatementIterator; ++innerStatementIterator)
+		    statements.end() != innerStatementIterator; ++innerStatementIterator)
 		{
 			if(outerStatementIterator == innerStatementIterator) continue;
 
 			auto& innerStatement(*innerStatementIterator);
-			
+
 			innerStatement.VisitSources(
-				[&] (SymbolRefPtr& symbol, bool)
-				{
-					if(symbol->Equals(outerStatement.dst.get()))
-					{
-						symbol = outerStatement.src1;
-						changed = true;
-					}
-				}
-			);
+			    [&](SymbolRefPtr& symbol, bool) {
+				    if(symbol->Equals(outerStatement.dst.get()))
+				    {
+					    symbol = outerStatement.src1;
+					    changed = true;
+				    }
+			    });
 		}
 	}
 	return changed;
@@ -1114,7 +1162,7 @@ bool CJitter::ReorderAdd(StatementList& statements)
 	bool changed = false;
 
 	for(auto statementIterator(statements.begin());
-		statements.end() != statementIterator; ++statementIterator)
+	    statements.end() != statementIterator; ++statementIterator)
 	{
 		auto& statement(*statementIterator);
 
@@ -1247,9 +1295,8 @@ bool CJitter::CopyPropagation(StatementList& statements)
 		// - t1 = r0 + 30   //innerStatement
 		//After substitution, t0 will not be used anymore making outerStatement eligible for removal
 		else if(
-			(outerStatement.op == innerStatement.op) && 
-			((innerStatement.op == OP_ADD) || (innerStatement.op == OP_ADDREF))
-			)
+		    (outerStatement.op == innerStatement.op) &&
+		    ((innerStatement.op == OP_ADD) || (innerStatement.op == OP_ADDREF)))
 		{
 			auto innerSrc2cst = dynamic_symbolref_cast(SYM_CONSTANT, innerStatement.src2);
 			auto outerSrc2cst = dynamic_symbolref_cast(SYM_CONSTANT, outerStatement.src2);
@@ -1266,6 +1313,169 @@ bool CJitter::CopyPropagation(StatementList& statements)
 	return changed;
 }
 
+bool CJitter::CommonExpressionElimination(VERSIONED_STATEMENT_LIST& versionedStatementList)
+{
+	bool changed = false;
+	std::vector<StatementList::const_iterator> tempDefs;
+	std::unordered_map<SymbolPtr, SymbolPtr> tempReplaceMap;
+	tempDefs.reserve(versionedStatementList.statements.size());
+
+	for(auto statementIterator(versionedStatementList.statements.begin());
+	    versionedStatementList.statements.end() != statementIterator; ++statementIterator)
+	{
+		auto& statement(*statementIterator);
+
+		//If this is a statement defining a temporary
+		if(
+		    (statement.op != OP_RETVAL) &&
+		    (statement.dst) &&
+		    (statement.dst->GetSymbol()->IsTemporary()))
+		{
+			const auto& newTemp = statement.dst->GetSymbol();
+
+			bool found = false;
+			//Check if our temporary already has a similar definition
+			for(const auto& tempDef : tempDefs)
+			{
+				const auto& tempDefStatement = *tempDef;
+				assert(tempDefStatement.dst);
+				const auto& temp = tempDefStatement.dst;
+				if(statement.op != tempDefStatement.op) continue;
+				if(statement.jmpCondition != tempDefStatement.jmpCondition) continue;
+				if(statement.src1 && !statement.src1->Equals(tempDefStatement.src1.get())) continue;
+				if(statement.src2 && !statement.src2->Equals(tempDefStatement.src2.get())) continue;
+				if(statement.src3 && !statement.src3->Equals(tempDefStatement.src3.get())) continue;
+				auto [_, inserted] = tempReplaceMap.insert(std::make_pair(newTemp, temp->GetSymbol()));
+				assert(inserted);
+				found = true;
+				//We assume the first replacement we find is gonna be the best
+				break;
+			}
+
+			//We haven't found a replacement for our definition, assume it's new
+			if(!found)
+			{
+				tempDefs.push_back(statementIterator);
+			}
+		}
+
+		if(tempReplaceMap.empty())
+		{
+			continue;
+		}
+
+		statement.VisitSources(
+		    [&](SymbolRefPtr& innerSymbolRef, bool) {
+			    if(!innerSymbolRef->GetSymbol()->IsTemporary()) return;
+			    if(auto tempReplaceIterator = tempReplaceMap.find(innerSymbolRef->GetSymbol()); tempReplaceIterator != std::end(tempReplaceMap))
+			    {
+				    innerSymbolRef = MakeSymbolRef(tempReplaceIterator->second);
+				    changed = true;
+			    }
+		    });
+	}
+
+	return changed;
+}
+
+bool CJitter::ClampingElimination(StatementList& statements)
+{
+	bool changed = false;
+
+	std::map<uint32, uint32> temporaryValues;
+
+	for(auto statementIterator(statements.begin());
+	    statements.end() != statementIterator; ++statementIterator)
+	{
+		auto& statement = *statementIterator;
+
+		auto dstSymbol = dynamic_symbolref_cast(SYM_TEMPORARY128, statement.dst);
+
+		if((statement.op == OP_MD_EXPAND_W) &&
+		   dstSymbol &&
+		   statement.src1->GetSymbol()->IsConstant())
+		{
+			auto src1Symbol = dynamic_symbolref_cast(SYM_CONSTANT, statement.src1);
+			temporaryValues.insert(
+			    std::make_pair(dstSymbol->m_valueLow, src1Symbol->m_valueLow));
+		}
+		else if(dstSymbol)
+		{
+			//Make sure that our temporary value doesn't get modified
+			assert(temporaryValues.find(dstSymbol->m_valueLow) == std::end(temporaryValues));
+		}
+
+		if((statement.op == OP_MD_CLAMP_S) &&
+		   statement.src1->GetSymbol()->IsTemporary())
+		{
+			auto src1Symbol = dynamic_symbolref_cast(SYM_TEMPORARY128, statement.src1);
+			auto tempCstValue = temporaryValues.find(src1Symbol->m_valueLow);
+			if(tempCstValue != std::end(temporaryValues))
+			{
+				if((tempCstValue->second & 0x7F800000) != 0x7F800000)
+				{
+					//No clamping needed here
+					statement.op = OP_MOV;
+					changed = true;
+				}
+			}
+		}
+	}
+
+	return changed;
+}
+
+bool CJitter::MergeCmpSelectOps(StatementList& statements)
+{
+	bool changed = false;
+
+	for(auto statementIterator(statements.begin());
+	    statements.end() != statementIterator; ++statementIterator)
+	{
+		auto& statement = *statementIterator;
+		auto nextStatementIterator = statementIterator;
+		nextStatementIterator++;
+
+		if(nextStatementIterator == std::end(statements)) continue;
+		auto& nextStatement = *nextStatementIterator;
+
+		if(nextStatement.op != OP_SELECT) continue;
+		if(statement.op != OP_CMP) continue;
+
+		auto predicateSymbol = statement.dst;
+
+		if(!nextStatement.src1->Equals(predicateSymbol.get())) continue;
+
+		auto innerStatementIterator = nextStatementIterator;
+		innerStatementIterator++;
+
+		//Make sure predicate isn't used later
+		bool used = false;
+		for(; innerStatementIterator != statements.end(); ++innerStatementIterator)
+		{
+			const auto& innerStatement = *innerStatementIterator;
+			innerStatement.VisitSources([&](const SymbolRefPtr& src, bool isDst) { used |= src->Equals(predicateSymbol.get()); });
+			if(used) break;
+		}
+
+		if(used) continue;
+
+		nextStatement.op = OP_CMPSELECT_P2;
+		nextStatement.jmpCondition = statement.jmpCondition;
+		nextStatement.src1 = nextStatement.src2;
+		nextStatement.src2 = nextStatement.src3;
+		nextStatement.src3.reset();
+
+		statement.op = OP_CMPSELECT_P1;
+		statement.jmpCondition = Jitter::CONDITION_NEVER;
+		statement.dst.reset();
+
+		changed = true;
+	}
+
+	return changed;
+}
+
 bool CJitter::DeadcodeElimination(VERSIONED_STATEMENT_LIST& versionedStatementList)
 {
 	bool changed = false;
@@ -1273,56 +1483,53 @@ bool CJitter::DeadcodeElimination(VERSIONED_STATEMENT_LIST& versionedStatementLi
 	typedef std::list<StatementList::iterator> ToDeleteList;
 	ToDeleteList toDelete;
 
-	for(StatementList::iterator outerStatementIterator(versionedStatementList.statements.begin());
-		versionedStatementList.statements.end() != outerStatementIterator; ++outerStatementIterator)
+	for(auto outerStatementIterator(versionedStatementList.statements.begin());
+	    versionedStatementList.statements.end() != outerStatementIterator; ++outerStatementIterator)
 	{
-		STATEMENT& outerStatement(*outerStatementIterator);
+		auto& outerStatement(*outerStatementIterator);
+		const auto& symbolRef(outerStatement.dst);
 
-		CSymbol* candidate = NULL;
-		if(outerStatement.dst && outerStatement.dst->GetSymbol()->IsTemporary())
+		CSymbol* candidate = nullptr;
+		if(symbolRef && symbolRef->GetSymbol()->IsTemporary())
 		{
-			candidate = outerStatement.dst->GetSymbol().get();
+			candidate = symbolRef->GetSymbol().get();
 		}
-		else if(CSymbol* relativeSymbol = dynamic_symbolref_cast(SYM_RELATIVE, outerStatement.dst))
+		else if(auto relativeSymbol = dynamic_symbolref_cast(SYM_RELATIVE, symbolRef))
 		{
-			VersionedSymbolRefPtr versionedSymbolRef = std::dynamic_pointer_cast<CVersionedSymbolRef>(outerStatement.dst);
-			assert(versionedSymbolRef);
-			if(versionedSymbolRef->version != versionedStatementList.relativeVersions.GetRelativeVersion(relativeSymbol->m_valueLow))
+			assert(symbolRef->IsVersioned());
+			if(symbolRef->GetVersion() != versionedStatementList.relativeVersions.GetRelativeVersion(relativeSymbol->m_valueLow))
 			{
 				candidate = relativeSymbol;
 			}
 		}
 
-		if(candidate == NULL) continue;
-		const SymbolRefPtr& symbolRef(outerStatement.dst);
+		if(!candidate) continue;
 
 		//Look for any possible use of this symbol
 		bool used = false;
-		for(StatementList::iterator innerStatementIterator(outerStatementIterator);
-			versionedStatementList.statements.end() != innerStatementIterator; ++innerStatementIterator)
+		for(auto innerStatementIterator(outerStatementIterator);
+		    versionedStatementList.statements.end() != innerStatementIterator; ++innerStatementIterator)
 		{
 			if(outerStatementIterator == innerStatementIterator) continue;
 
 			const auto& innerStatement(*innerStatementIterator);
-			
-			innerStatement.VisitSources(
-				[&](const SymbolRefPtr& innerSymbolRef, bool)
-				{
-					if(innerSymbolRef->Equals(symbolRef.get()))
-					{
-						used = true;
-						return;
-					}
 
-					auto symbol(innerSymbolRef->GetSymbol());
-					if(!symbol->Equals(candidate) && symbol->Aliases(candidate))
-					{
-						used = true;
-						return;
-					}
-				}
-			);
-			
+			innerStatement.VisitSources(
+			    [&](const SymbolRefPtr& innerSymbolRef, bool) {
+				    if(innerSymbolRef->Equals(symbolRef.get()))
+				    {
+					    used = true;
+					    return;
+				    }
+
+				    auto symbol(innerSymbolRef->GetSymbol());
+				    if(!symbol->Equals(candidate) && symbol->Aliases(candidate))
+				    {
+					    used = true;
+					    return;
+				    }
+			    });
+
 			if(used) break;
 		}
 
@@ -1334,7 +1541,7 @@ bool CJitter::DeadcodeElimination(VERSIONED_STATEMENT_LIST& versionedStatementLi
 	}
 
 	for(ToDeleteList::const_iterator deleteIterator(toDelete.begin());
-		toDelete.end() != deleteIterator; ++deleteIterator)
+	    toDelete.end() != deleteIterator; ++deleteIterator)
 	{
 		versionedStatementList.statements.erase(*deleteIterator);
 		changed = true;
@@ -1349,7 +1556,7 @@ void CJitter::CoalesceTemporaries(BASIC_BLOCK& basicBlock)
 	EncounteredTempList encounteredTemps;
 
 	for(auto outerStatementIterator(basicBlock.statements.begin());
-		basicBlock.statements.end() != outerStatementIterator; ++outerStatementIterator)
+	    basicBlock.statements.end() != outerStatementIterator; ++outerStatementIterator)
 	{
 		auto& outerStatement(*outerStatementIterator);
 
@@ -1368,22 +1575,20 @@ void CJitter::CoalesceTemporaries(BASIC_BLOCK& basicBlock)
 			bool used = false;
 
 			for(auto innerStatementIterator(outerStatementIterator);
-				basicBlock.statements.end() != innerStatementIterator; ++innerStatementIterator)
+			    basicBlock.statements.end() != innerStatementIterator; ++innerStatementIterator)
 			{
 				if(outerStatementIterator == innerStatementIterator) continue;
 
 				const auto& innerStatement(*innerStatementIterator);
 				innerStatement.VisitOperands(
-					[&](const SymbolRefPtr& symbolRef, bool)
-					{
-						auto symbol = symbolRef->GetSymbol();
-						if(symbol->Equals(encounteredTemp))
-						{
-							used = true;
-						}
-					}
-				);
-				
+				    [&](const SymbolRefPtr& symbolRef, bool) {
+					    auto symbol = symbolRef->GetSymbol();
+					    if(symbol->Equals(encounteredTemp))
+					    {
+						    used = true;
+					    }
+				    });
+
 				if(used) break;
 			}
 
@@ -1406,7 +1611,7 @@ void CJitter::CoalesceTemporaries(BASIC_BLOCK& basicBlock)
 
 			//Replace all occurences of this temp with the candidate
 			for(auto innerStatementIterator(outerStatementIterator);
-				basicBlock.statements.end() != innerStatementIterator; ++innerStatementIterator)
+			    basicBlock.statements.end() != innerStatementIterator; ++innerStatementIterator)
 			{
 				if(outerStatementIterator == innerStatementIterator) continue;
 
@@ -1452,7 +1657,7 @@ void CJitter::CoalesceTemporaries(BASIC_BLOCK& basicBlock)
 void CJitter::RemoveSelfAssignments(BASIC_BLOCK& basicBlock)
 {
 	for(StatementList::iterator outerStatementIterator(basicBlock.statements.begin());
-		basicBlock.statements.end() != outerStatementIterator; )
+	    basicBlock.statements.end() != outerStatementIterator;)
 	{
 		STATEMENT& outerStatement(*outerStatementIterator);
 
@@ -1475,16 +1680,14 @@ void CJitter::PruneSymbols(BASIC_BLOCK& basicBlock) const
 	for(const auto& statement : basicBlock.statements)
 	{
 		statement.VisitOperands(
-			[&] (const SymbolRefPtr& symbolRef, bool)
-			{
-				const auto& symbol = symbolRef->GetSymbol();
-				symbolUseCount[symbol.get()]++;
-			}
-		);
+		    [&](const SymbolRefPtr& symbolRef, bool) {
+			    const auto& symbol = symbolRef->GetSymbol();
+			    symbolUseCount[symbol.get()]++;
+		    });
 	}
 
 	for(auto symbolIterator(std::begin(symbolTable.GetSymbols()));
-		symbolIterator != std::end(symbolTable.GetSymbols());)
+	    symbolIterator != std::end(symbolTable.GetSymbols());)
 	{
 		const auto& symbol(*symbolIterator);
 		if(symbolUseCount.find(symbol.get()) == std::end(symbolUseCount))
@@ -1503,7 +1706,7 @@ unsigned int CJitter::AllocateStack(BASIC_BLOCK& basicBlock)
 	unsigned int stackAlloc = 0;
 	for(const auto& symbol : basicBlock.symbolTable.GetSymbols())
 	{
-		if(symbol->m_type == SYM_TEMPORARY || symbol->m_type == SYM_FP_TMP_SINGLE)
+		if((symbol->m_type == SYM_TEMPORARY) || (symbol->m_type == SYM_FP_TEMPORARY32))
 		{
 			symbol->m_stackLocation = stackAlloc;
 			stackAlloc += 4;
@@ -1556,7 +1759,7 @@ unsigned int CJitter::AllocateStack(BASIC_BLOCK& basicBlock)
 
 void CJitter::NormalizeStatements(BASIC_BLOCK& basicBlock)
 {
-	//Reorganize the commutative statements 
+	//Reorganize the commutative statements
 	//1. Always have registers as the first operand
 	//2. Always have constants as the last operand
 
@@ -1567,46 +1770,46 @@ void CJitter::NormalizeStatements(BASIC_BLOCK& basicBlock)
 
 		switch(statement.op)
 		{
-			case OP_ADD:
-			case OP_ADD64:
-			case OP_AND:
-			case OP_AND64:
-			case OP_OR:
-			case OP_XOR:
-			case OP_MUL:
-			case OP_MULS:
-			case OP_MD_AND:
-			case OP_MD_OR:
-			case OP_MD_XOR:
-			case OP_MD_ADD_B:
-			case OP_MD_ADD_H:
-			case OP_MD_ADD_W:
-			case OP_MD_ADDSS_H:
-			case OP_MD_ADDSS_W:
-			case OP_MD_ADDUS_B:
-			case OP_MD_ADDUS_W:
-			case OP_MD_CMPEQ_B:
-			case OP_MD_CMPEQ_H:
-			case OP_MD_CMPEQ_W:
-			case OP_MD_MIN_H:
-			case OP_MD_MIN_W:
-			case OP_MD_MAX_H:
-			case OP_MD_MAX_W:
-			case OP_MD_ADD_S:
-			case OP_MD_MUL_S:
-			case OP_MD_MIN_S:
-			case OP_MD_MAX_S:
-				isCommutative = true;
-				break;
-			case OP_CMP:
-			case OP_CMP64:
-			case OP_CONDJMP:
-				isCommutative = true;
-				conditionSwapRequired = true;
-				break;
-			default:
-				isCommutative = false;
-				break;
+		case OP_ADD:
+		case OP_ADD64:
+		case OP_AND:
+		case OP_AND64:
+		case OP_OR:
+		case OP_XOR:
+		case OP_MUL:
+		case OP_MULS:
+		case OP_MD_AND:
+		case OP_MD_OR:
+		case OP_MD_XOR:
+		case OP_MD_ADD_B:
+		case OP_MD_ADD_H:
+		case OP_MD_ADD_W:
+		case OP_MD_ADDSS_H:
+		case OP_MD_ADDSS_W:
+		case OP_MD_ADDUS_B:
+		case OP_MD_ADDUS_W:
+		case OP_MD_CMPEQ_B:
+		case OP_MD_CMPEQ_H:
+		case OP_MD_CMPEQ_W:
+		case OP_MD_MIN_H:
+		case OP_MD_MIN_W:
+		case OP_MD_MAX_H:
+		case OP_MD_MAX_W:
+		case OP_MD_ADD_S:
+		case OP_MD_MUL_S:
+		case OP_MD_MIN_S:
+		case OP_MD_MAX_S:
+			isCommutative = true;
+			break;
+		case OP_CMP:
+		case OP_CMP64:
+		case OP_CONDJMP:
+			isCommutative = true;
+			conditionSwapRequired = true;
+			break;
+		default:
+			isCommutative = false;
+			break;
 		}
 
 		if(!isCommutative) continue;
@@ -1627,7 +1830,7 @@ void CJitter::NormalizeStatements(BASIC_BLOCK& basicBlock)
 
 		//Check if register operand is at the end and swap if it is the case
 		{
-			bool dstreg  = statement.dst && statement.dst->GetSymbol()->IsRegister();
+			bool dstreg = statement.dst && statement.dst->GetSymbol()->IsRegister();
 			bool src1reg = statement.src1->GetSymbol()->IsRegister();
 			bool src2reg = statement.src2->GetSymbol()->IsRegister();
 
@@ -1636,8 +1839,8 @@ void CJitter::NormalizeStatements(BASIC_BLOCK& basicBlock)
 				std::swap(statement.src1, statement.src2);
 				swapped = true;
 			}
-			else if(dstreg && src1reg && src2reg && 
-				statement.dst->GetSymbol()->Equals(statement.src2->GetSymbol().get()))
+			else if(dstreg && src1reg && src2reg &&
+			        statement.dst->GetSymbol()->Equals(statement.src2->GetSymbol().get()))
 			{
 				//If all operands are registers and dst is equal to src2, swap to make dst and src2 side by side
 				std::swap(statement.src1, statement.src2);
@@ -1655,15 +1858,27 @@ void CJitter::NormalizeStatements(BASIC_BLOCK& basicBlock)
 			case CONDITION_BL:
 				statement.jmpCondition = CONDITION_AB;
 				break;
+			case CONDITION_BE:
+				statement.jmpCondition = CONDITION_AE;
+				break;
 			case CONDITION_AB:
 				statement.jmpCondition = CONDITION_BL;
 				break;
+			case CONDITION_AE:
+				statement.jmpCondition = CONDITION_BE;
+				break;
 			case CONDITION_LT:
 				statement.jmpCondition = CONDITION_GT;
 				break;
+			case CONDITION_LE:
+				statement.jmpCondition = CONDITION_GE;
+				break;
 			case CONDITION_GT:
 				statement.jmpCondition = CONDITION_LT;
 				break;
+			case CONDITION_GE:
+				statement.jmpCondition = CONDITION_LE;
+				break;
 			default:
 				assert(0);
 				break;
